#ifndef _DaHa_TreePartition_
#define _DaHa_TreePartition_

template <typename box_id_type>
class BoxRange
{
    using BoxID = box_id_type;
    using SfcIndexType = typename BoxID::sfcindex_type;
    using depth_type = typename BoxID::depth_type;

    using size_type = SfcIndexType;

    class BoxIterator
    {
    public:
        BoxIterator(SfcIndexType pos, depth_type d) : pos_(pos), d_(d){}
        bool operator==(const BoxIterator &other) const
        {
            return other.pos_ == pos_ && other.d_ == d_;
        }
        bool operator!=(const BoxIterator &other) const
        {
            return !operator==(other);
        }
        BoxIterator &operator++()
        {
            // FIXME depth increment, only on one depth at the moment
            pos_++;
            return *this;
        }
        SfcIndexType pos() const
        {
            return pos_;
        }
        depth_type d() const
        {
            return d_;
        }

        BoxID operator*() const
        {
            return BoxID(pos_, d_);
        }

    private:
        SfcIndexType pos_;
        depth_type d_;
    };


public:
    BoxRange(SfcIndexType begin, SfcIndexType end, depth_type d)
            : begin_(begin, d), end_(end, d), d_(d){}
    BoxIterator begin() const
    {
        return begin_;
    }
    BoxIterator end() const
    {
        return end_;
    }

    size_type size() const
    {
        return end_.pos() - begin_.pos();
    }


private:
    BoxIterator begin_, end_;
    depth_type d_;
};

template <typename box_id_type>
class TreePartition
{
    using BoxID = box_id_type;
    using SfcIndexType = typename BoxID::sfcindex_type;
    using depth_type = typename BoxID::depth_type;
    using BoxRangeType = BoxRange<BoxID>;

public:
    TreePartition(SfcIndexType begin, SfcIndexType end, depth_type d)
            : begin_(begin), end_(end), d_(d){}
    TreePartition(SfcIndexType id, depth_type d) : begin_(id), end_(id + 1), d_(d){}

    bool operator==(const TreePartition &other) const
    {
        return other.begin_ == begin_ && other.end_ == end_;
    }
    bool operator!=(const TreePartition &other) const
    {
        return !operator==(other);
    }

    bool empty() const
    {
        return begin_ == end_;
    }

    BoxRangeType box_range() const
    {
        return BoxRangeType(begin_, end_, d_);
    }

    BoxID first() const
    {
        return BoxID(begin_, d_);
    }

private:
    SfcIndexType begin_, end_;
    depth_type d_;
};

#endif
