#!/usr/bin/perl -w
#
# This file is part of FMSOLVR, a Fast Multipole Solver for MD.
#
# Copyright © 2019 Andreas Beckmann
#
# FMSOLVR is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# FMSOLVR is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with FMSOLVR.  If not, see
# <https://www.gnu.org/licenses/>.
#

$file = shift;
$simd = shift || 1;

$p = 0;
$ws = 0;

%A = ();

print "# counter p ws total amortized\n";

open IN, $file;
while (<IN>)
{
        chomp;
	if (/^p=(\d+)  ws=(\d+)/) {
		$p = int($1);
		$ws = int($2);
	} elsif (/^COUNTER\((.*)\)=(.*) per step: (\S*)/) {
		printf("%-48s %7d %7d %15s %15s\n", $1, $p, $ws, $2, $3);
		$A{$ws}{$1}{$p} = [$2, $3];
	}
}

print "\n";

my @ws = sort {$a <=> $b} keys %A;
#my %nm = map { $_, 1 } keys %{$A{$ws}} foreach $ws (@ws);
#my %nm = map { $_, 1 } map { keys %{$A{$_}} } @ws;
#my @nm = sort keys %nm;
##my %p = map { $_, 1 } map { keys %{$A{$_}} } @ws;
##my @p = sort keys %p;

my @nm = sort keys %{$A{$ws[0]}};
my @p = sort {$a <=> $b} keys %{$A{$ws[0]}{$nm[0]}};

foreach $ws (@ws) {
	printf "# ws=%-40d p=", $ws;
	foreach $p (@p) {
		printf " %15d", $p;
	}
	print "\n";
	foreach $nm (@nm) {
		@vals = reverse sort {$a <=> $b} map { $A{$ws}{$nm}{$_}[1] } @p;
		next if $vals[0] == 0;
		printf "%-48s", $nm;
		foreach $p (@p) {
			printf " %15g", $A{$ws}{$nm}{$p}[1] * $simd;
		}
		print "   $nm\n";
	}
}

print "\n";

my @counters = ("PAPI_TOT_INS", "PAPI_TOT_CYC");
printf "# %6s %7s", "ws", "p";
foreach $c (@counters) {
	printf " %15s", $c;
}
print "\n";
foreach $ws (@ws) {
	foreach $p (@p) {
		printf "%8d %7d", $ws, $p;
		foreach $c (@counters) {
			printf " %15s", $A{$ws}{$c}{$p}[0];
		}
		print "\n";
	}
}
