// add local coefficients for a particle with charge q at relative position (x,y,z)
// to the (non-negative triangle) local coefficient matrix upto order p
template <typename Real, typename CoefficientMatrix>
inline void P2L(
        const Real x,
        const Real y,
        const Real z,
        const Real q,
        CoefficientMatrix & mu,
        const size_t p)
{
    typedef Real real_type;
    typedef typename CoefficientMatrix::value_type complex_type;
    Vec_traits<Real> VT;

    real_type dist_squared = x * x + y * y + z * z;
    real_type recip_dist = rsqrt(dist_squared);
    real_type recip_dist_squared = reciprocal(dist_squared);
    real_type z_recip_dist_squared = z * recip_dist_squared;

    complex_type complex_x_y(x, y);

    // mu_0_0  (for the first iteration)
    complex_type complex_part(VT.same(1));
    real_type mu_mplus1_mplus1 = q * recip_dist;

    // mu_0_0 upto mu_p_p-1
    for (size_t m = 0; m < p; ++m) {
        // mu_m_m  (from previous iteration)
        real_type mu_m_m = mu_mplus1_mplus1;
        mu(m, m) += complex_part * mu_m_m;

        // mu_m+1_m+1  (for the next iteration)
        mu_mplus1_mplus1 = VT.same(2 * m + 1) * recip_dist_squared * mu_m_m;

        // mu_m+1_m
        real_type mu_mplus1_m = (VT.same(2 * m + 1) * recip_dist_squared * mu_m_m) * z;
        mu(m + 1, m) += complex_part * mu_mplus1_m;

        // mu_m+2_m upto mu_p_m
        real_type mu_lminus2_m = mu_m_m;
        real_type mu_lminus1_m = mu_mplus1_m;
        for (size_t l = m + 2; l <= p; ++l) {
            // mu_l_m
            real_type mu_l_m =
                VT.same(2 * l - 1) * z_recip_dist_squared * mu_lminus1_m
                - VT.same((l - 1) * (l - 1) - m * m) * recip_dist_squared * mu_lminus2_m;
            mu(l, m) += complex_part * mu_l_m;
            mu_lminus2_m = mu_lminus1_m;
            mu_lminus1_m = mu_l_m;
        }

        // (for the next iteration)
        complex_part *= complex_x_y;
    }
    // mu_p_p
    mu(p, p) += complex_part * mu_mplus1_mplus1;
}

