#
# This file is part of FMSOLVR, a Fast Multipole Solver for MD.
#
# Copyright © 2019 Andreas Beckmann
#
# FMSOLVR is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# FMSOLVR is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with FMSOLVR.  If not, see
# <https://www.gnu.org/licenses/>.
#
GTEST_SRC		?= /usr/src/gtest
UT_DIRS			 = unittests
UT_DIRS			+= unittests/parallelization
TEST_DIRS		?= $(UT_DIRS)

GTEST_OBJS	 = unittests/gtest-all.o unittests/gtest_main.o

$(GTEST_OBJS): | unittests

unittests/gtest-all.o: $(GTEST_SRC)/src/gtest-all.cc $(EXTRADEPS)
	$(COMPILE.cc) -I$(GTEST_SRC) -I$(GTEST_SRC)/include $(OUTPUT_OPTION) $<

unittests/gtest_main.o: $(GTEST_SRC)/src/gtest_main.cc $(EXTRADEPS)
	$(COMPILE.cc) -I$(GTEST_SRC) -I$(GTEST_SRC)/include $(OUTPUT_OPTION) $<

build-unittests::
run-unittests:: build-unittests
clean-unittests::
	$(RM) $(GTEST_OBJS) $(GTEST_OBJS:.o=.d)

# $1 = dir
define ut_target
GTEST_EXECUTABLE_$1	 = $1/run
TEST_SOURCES_$1		 = $$(wildcard $$(SRCROOT)/$1/*.cpp)
TEST_OBJECTS_$1		 = $$(patsubst $$(SRCROOT)/%,$$(OBJROOT)/%,$$(TEST_SOURCES_$1:.cpp=.o))

$$(TEST_OBJECTS_$1): CPPFLAGS += -I$(GTEST_SRC)/include
$$(TEST_OBJECTS_$1): $$(EXTRADEPS)
$$(TEST_OBJECTS_$1): | $1

$1:
	mkdir -p $$@

$$(GTEST_EXECUTABLE_$1): $$(TEST_OBJECTS_$1) $$(GTEST_OBJS)
	$$(LINK.o) $$^ $$(LOADLIBES) $$(LDLIBS) -o $$@

build-unittests:: $$(GTEST_EXECUTABLE_$1)

run-unittests::
	$$(GTEST_EXECUTABLE_$1) --gtest_color=yes

clean-unittests::
	$$(RM) $$(GTEST_EXECUTABLE_$1) $1/*.o $1/*.d
endef

$(foreach d,$(TEST_DIRS),$(eval $(call ut_target,$d)))

check: run-unittests

check-%:
	$(MAKE) check TEST_DIRS=unittests/$*

-include $(foreach d,$(TEST_DIRS),$d/*.d)

clean:: clean-unittests

############################################################

not_equal	 = $(filter-out $1,$2)
is_absolute	 = $(filter /%,$1)

# $1 = relative distance
update_srcroot	 = $(if $(call is_absolute,$(SRCROOT)),$(SRCROOT),$1/$(SRCROOT))

# $1 = subdir, $2 = ..
subdirflags = -C $1 $(if $(call not_equal,$(SRCROOT),$(OBJROOT)),-f $(call update_srcroot,$2)/$1/Makefile SRCROOT=$(call update_srcroot,$2))

s-c-h: | self-contained-headers
	$(MAKE) $(call subdirflags,self-contained-headers,..) all

clean:: | self-contained-headers
	$(MAKE) $(call subdirflags,self-contained-headers,..) clean

self-contained-headers:
	mkdir -p $@
