#
# This file is part of FMSOLVR, a Fast Multipole Solver for MD.
#
# Copyright © 2019 Andreas Beckmann
#
# FMSOLVR is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# FMSOLVR is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with FMSOLVR.  If not, see
# <https://www.gnu.org/licenses/>.
#

OBJROOT		?= .
SRCROOT		?= $(OBJROOT)

HOSTNAME	:= $(shell hostname)
SYSTEMNAME	:= $(shell if [ -f /etc/FZJ/systemname ]; then cat /etc/FZJ/systemname ; fi)

-include $(SRCROOT)/Makefile.$(HOSTNAME)
-include $(SRCROOT)/Makefile.$(SYSTEMNAME)

#PLATFORM	?= x86_64
#PLATFORM	?= armv6l
#PLATFORM	?= armv7l
#PLATFORM	?= ppc64
PLATFORM	?= $(shell uname -m)

COMPILER	?= gnu
COMPILER	?= clang
COMPILER	?= pgi

include $(SRCROOT)/build-$(COMPILER).mk
include $(SRCROOT)/build-$(PLATFORM).mk

############################################################################

WITH_OPENMP	?= $(HAS_OPENMP)
WITH_STD_THREAD	?= $(HAS_STD_THREAD)
WITH_BOOST	?= $(and $(COMPILER_HAS_BOOST),$(PLATFORM_HAS_BOOST))
WITH_PAPI	?= $(HAS_PAPI)
WITH_QUADMATH	?= $(HAS_QUADMATH_$(PLATFORM))

GITBRANCH	:= $(shell cd $(SRCROOT) && git rev-parse --abbrev-ref HEAD)
GITREVISION	:= $(shell cd $(SRCROOT) && git describe --always --long --tags --dirty)

CPPFLAGS	+= -I$(SRCROOT)/include
CPPFLAGS	+= -I$(SRCROOT)/tables
CPPFLAGS	+= -I$(SRCROOT)/helpers
CPPFLAGS	+= -I$(SRCROOT)
CPPFLAGS	+= -DGITBRANCH="\"$(GITBRANCH)\"" -DGITREVISION="\"$(GITREVISION)\""
CPPFLAGS	+= -DCOMPILER="\"$(CXX)\""
CPPFLAGS	+= -DCOMPILER_VERSION="\"$(COMPILER_VERSION)\""
CPPFLAGS	+= -DCOMPILER_FLAGS="\"$(CXXFLAGS)\""

STD_FLAG	?= -std=c++11
OPENMP_FLAG	?= -fopenmp
PTHREAD_FLAG	?= -pthread
ABI_FLAG	?=
AS_NEEDED_FLAG	?= -Wl,--as-needed
OPTLEVEL	?= 3

CXXFLAGS_OPTIMIZE	?= -O$(OPTLEVEL)
CXXFLAGS_WARN		?= -W -Wall
CXXFLAGS_DEBUG		?= -g
CXXFLAGS_DEPS		?= -MD

CXXFLAGS	+= $(STD_FLAG)
CXXFLAGS	+= $(CXXFLAGS_OPTIMIZE)
CXXFLAGS	+= $(CXXFLAGS_ARCH)
CXXFLAGS	+= $(CXXFLAGS_WARN)
CXXFLAGS	+= $(CXXFLAGS_DEBUG)
CXXFLAGS	+= $(CXXFLAGS_DEPS)
CXXFLAGS	+= $(PTHREAD_FLAG)
CXXFLAGS	+= $(ABI_FLAG)

LDFLAGS		+= $(AS_NEEDED_FLAG)

ifneq (,$(WITH_OPENMP))
CPPFLAGS	+= -DWITH_OPENMP
CXXFLAGS	+= $(OPENMP_FLAG)
endif

ifneq (,$(WITH_STD_THREAD))
CPPFLAGS	+= -DWITH_STD_THREAD
endif

ifneq (,$(WITH_BOOST))
CPPFLAGS	+= -DWITH_BOOST
LDLIBS		+= -lboost_thread
LDLIBS		+= -lboost_system
endif

ifneq (,$(WITH_PAPI))
ifneq (,$(PAPI_ROOT))
CPPFLAGS	+= -I$(PAPI_ROOT)/include
LDLIBS		+= -L$(PAPI_ROOT)/lib
LDLIBS		+= -Wl,-rpath $(PAPI_ROOT)/lib
endif
CPPFLAGS	+= -DWITH_PAPI
LDLIBS		+= -lpapi
endif

ifneq (,$(WITH_QUADMATH))
CXXFLAGS	+= -fext-numeric-literals
LDLIBS		+= -lquadmath
endif

CPPFLAGS	+= $(CPPFLAGS.$(HOSTNAME))
CPPFLAGS	+= $(CPPFLAGS.$(SYSTEMNAME))

CXXFLAGS	+= $(CXXFLAGS.$(HOSTNAME))
CXXFLAGS	+= $(CXXFLAGS.$(SYSTEMNAME))

LDFLAGS		+= $(LDFLAGS.$(HOSTNAME))
LDFLAGS		+= $(LDFLAGS.$(SYSTEMNAME))

LINK.o		 = $(CXX) $(CXXFLAGS) $(LDFLAGS) $(TARGET_ARCH)

%.h::
	@echo "Ignoring missing header $@..."
%.hpp::
	@echo "Ignoring missing header $@..."

vpath %.cpp $(SRCROOT)
vpath %.cc $(SRCROOT)
