/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <vector>

#include "fmsolvr/util/float128.hpp"
#include "fmsolvr/xyz.hpp"
#include "read_qxyz.hpp"
#include "read_fphi.hpp"
#include "compare_with_reference.hpp"

using namespace fmsolvr;

int main(int argc, char ** argv)
{
    if (argc != 4)
    {
        std::cout << "Usage: " << argv[0] << " input output reference" << std::endl;
        return 0;
    }

#if defined(HAVE_FLOAT128)
    typedef __float128 RefReal;
#else
    typedef long double RefReal;
#endif
    typedef XYZ<RefReal> RefReal3;
    typedef std::vector<RefReal> Vector1;
    typedef std::vector<RefReal3> Vector3;

    Vector1 q;
    Vector3 xyz;
    Vector1 Phi;
    Vector3 F;
    read_qxyz<RefReal>(argv[1], q, xyz);
    read_fphi<RefReal>(argv[2], F, Phi);
    RefReal Ec = 0.;
    for (size_t i = 0; i < q.size(); ++i)
        Ec += q[i] * Phi[i];
    Ec /= 2.;

    compare_with_reference<RefReal>(q, F, Phi, Ec, argv[3]);
}

// vim: et:ts=4:sw=4
