#!/usr/bin/perl
#
# This file is part of FMSOLVR, a Fast Multipole Solver for MD.
#
# Copyright © 2019 Andreas Beckmann
#
# FMSOLVR is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# FMSOLVR is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with FMSOLVR.  If not, see
# <https://www.gnu.org/licenses/>.
#


sub A($$$$$)
{
	my ($l, $m, $j, $k, $p) = @_;
	my $x = $l - $j;
	my $y = $m - $k;

	if (0 <= $x && $x <= $p && -$x <= $y && $y <= $x)
	{
		return 1;
	}
	return 0;
}

sub B($$$$$)
{
	my ($l, $m, $j, $k, $p) = @_;
	my $x = $l + $j;
	my $y = $m + $k;

	if (0 <= $x && $x <= 2*$p && -$x <= $y && $y <= $x)
	{
		return 1;
	}
	return 0;
}

sub C($$$$$)
{
	my ($l, $m, $j, $k, $p) = @_;
	my $x = $j - $l;
	my $y = $k - $m;

	if (0 <= $x && $x <= $p && -$x <= $y && $y <= $x)
	{
		return 1;
	}
	return 0;
}

print "# p+1\tops\tA\tB\tC\n";
my ($l, $m, $j, $k, $p);
foreach $p (0..50) {
	my $ops = 0;
	my $a = 0;
	my $b = 0;
	my $c = 0;
	foreach $l (0..$p) {
		foreach $m (-$l..$l) {
			foreach $j (0..$p) {
				foreach $k (-$j..$j) {
					++$ops;
					$a += A($l, $m, $j, $k, $p);
					$b += B($l, $m, $j, $k, $p);
					$c += C($l, $m, $j, $k, $p);
				}
			}
		}
	}
	printf("%d\t$ops\t$a\t$b\t$c\n", $p + 1);
}
