/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#ifndef _AnBe_FMM_make_grid_omega_hpp_
#define _AnBe_FMM_make_grid_omega_hpp_

#include "fmsolvr/operator/particle2multipole.hpp"
#include "fmsolvr/xyz.hpp"


namespace fmsolvr {

template <typename Omega, typename Real>
void make_grid_omega(int igrid, Real boxlength, Real charge, Omega &omega, int p)
{
    typedef XYZ<Real> Real3;
    Real q = charge / Real(igrid * igrid * igrid);
    Real gridstep = boxlength / Real(igrid - 1);
    Real3 center = Real3(0.5, 0.5, 0.5) * boxlength;
    for (int gi = 0; gi < igrid; ++gi)
    {
        for (int gj = 0; gj < igrid; ++gj)
        {
            for (int gk = 0; gk < igrid; ++gk)
            {
                P2M_nodivtable(q, Real3(gi, gj, gk) * gridstep, omega, center, p);
            }
        }
    }
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
