/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#ifndef _AnBe_FMM_read_fphi_hpp_
#define _AnBe_FMM_read_fphi_hpp_

#include <iostream>
#include <fstream>
#include <cstdlib>

namespace fmsolvr {

template <typename Real, typename VectorFxyz, typename VectorPhi>
void read_fphi(const char * f, VectorFxyz & vF, VectorPhi & vPhi)
{
    typedef typename VectorFxyz::value_type Real3;
    try {
        std::ifstream in(f);
        if (in.fail())
            throw "ioerror";
        Real x, y, z, phi;
        while (!in.eof()) {
            in >> x;
            if (in.eof())
                break;
            in >> y >> z >> phi;
            vF.push_back(Real3(x, y, z));
            vPhi.push_back(phi);
        }
    }
    catch (...)
    {
        std::cerr << "I/O error while reading input" << std::endl;
        exit(1);
    }
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
