/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#ifndef _AnBe_FMM_write_fphi_hpp_
#define _AnBe_FMM_write_fphi_hpp_

#include <iostream>
#include <fstream>
#include <iomanip>
#include <limits>
#include <cstdlib>

namespace fmsolvr {

template <typename Real>
struct write_fphi_precision {
    static const unsigned precision = std::numeric_limits<Real>::max_digits10 + 1;
};

template <typename Real, typename VectorFxyz, typename VectorPhi>
void write_fphi(const char * f, const VectorFxyz & vF, const VectorPhi & vPhi)
{
    try {
        std::ofstream out(f);
        if (out.fail())
            throw "ioerror";
        out << std::scientific << std::setprecision(write_fphi_precision<Real>::precision);
        for (size_t i = 0; i < vF.size(); ++i) {
            out
                << vF[i].x()
                << "\t" << vF[i].y()
                << "\t" << vF[i].z()
                << "\t" << vPhi[i]
                << std::endl;
        }
    }
    catch (...)
    {
        std::cerr << "I/O error while writing output" << std::endl;
        exit(1);
    }
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
