/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_CONTAINER_containercube_hpp
#define FMSOLVR_CONTAINER_containercube_hpp

#include <cstddef>


namespace fmsolvr {

template <typename Container>
class ContainerCube
{
public:
    typedef Container value_type;
    typedef const value_type * const_pointer;
    typedef const value_type & const_reference;
    typedef size_t size_type;

    ContainerCube(size_type dim) : m_dim(dim)
    {
        size_type size = m_dim * m_dim * m_dim;
        m_ptr = new const_pointer[size];
    }

    ~ContainerCube()
    {
        delete[] m_ptr;
    }

    void set(size_type i, size_type j, size_type k, const_reference container)
    {
        m_ptr[index(i, j, k)] = &container;
    }

    const_reference operator () (size_type i, size_type j, size_type k) const
    {
        return *m_ptr[index(i, j, k)];
    }

private:
    size_type index(size_type i, size_type j, size_type k) const
    {
        return (i * m_dim + j) * m_dim + k;
    }

    size_type m_dim;
    const_pointer * m_ptr;
};

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
