/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_CONTAINER_linear_array_hpp
#define FMSOLVR_CONTAINER_linear_array_hpp

#include "fmsolvr/container/simple_vector.hpp"

namespace fmsolvr
{

template <typename T, typename Allocator = std::allocator<T> >
class LinearArray : public simple_vector<T, Allocator>
{
    typedef simple_vector<T, Allocator> Base;

public:
    using typename Base::size_type;
    using typename Base::value_type;

    LinearArray(const size_t p_max)
        : Base(p_max + 1)
    { }

    size_type p() const
    { return size() - 1; }

private:
    size_type size() const
    { return Base::size(); }
};

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
