/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_CONTAINER_triangular_array_adaptor_hpp
#define FMSOLVR_CONTAINER_triangular_array_adaptor_hpp

namespace fmsolvr
{

// or generic?
template <typename Storage, typename Value, template <typename, typename> class Operation>
class simple_triangular_array_adaptor
{
    typedef Storage storage_type;
    typedef typename storage_type::value_type storage_value_type;
    typedef Value compute_value_type;
    typedef Operation<storage_value_type, compute_value_type> Op;
public:
    typedef typename storage_type::index_type index_type;
    typedef compute_value_type value_type;

    simple_triangular_array_adaptor(storage_type & storage)
        : storage(storage)
    { }

    void operator () (index_type l, index_type m, const compute_value_type & value) const
    {
        const Op op;
        op(storage.upper(l, m), value);
    }

private:
    storage_type & storage;
};


template <typename Operation>
class custom_triangular_array_adaptor
{
    typedef typename Operation::first_argument_type storage_type;
    typedef typename Operation::second_argument_type compute_value_type;
    typedef Operation Op;
public:
    typedef typename Operation::index_type index_type;
    typedef compute_value_type value_type;

    custom_triangular_array_adaptor(storage_type & storage, const Op & op = Op())
        : storage(storage), op(op)
    { }

    template <typename... Args>
    custom_triangular_array_adaptor(storage_type & storage, Args... args)
        : custom_triangular_array_adaptor(storage, Op(args...))
    { }

    void operator () (index_type l, index_type m, const compute_value_type & value)
    {
        op(storage, l, m, value);
    }

    Op & get_op()
    {
        return op;
    }

private:
    storage_type & storage;
    Op op;
};

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
