/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_CONTAINER_vector_permutation_adaptor_hpp
#define FMSOLVR_CONTAINER_vector_permutation_adaptor_hpp

namespace fmsolvr {

template <typename Vector, typename Permutation>
class vector_permutation_adaptor
{
    typedef typename Permutation::size_type size_type;
    typedef typename Vector::reference reference;
    typedef typename Vector::const_reference const_reference;

    Vector & v;
    const Permutation & p;

public:
    vector_permutation_adaptor(Vector & v, const Permutation & p) : v(v), p(p)
    {
        assert(v.size() == p.size());
    }

    const_reference operator [] (size_type i) const
    {
        return v[p[i]];
    }

    reference operator [] (size_type i)
    {
        return v[p[i]];
    }

    size_type size() const
    {
        return v.size();
    }

};

template <typename Vector, typename Permutation>
vector_permutation_adaptor<Vector, Permutation>
make_vector_permutation_adaptor(Vector & v, const Permutation & p)
{
    return vector_permutation_adaptor<Vector, Permutation>(v, p);
}


}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
