/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_farfield_interaction_hpp
#define FMSOLVR_farfield_interaction_hpp

#include <cstddef>
#include <utility>

namespace fmsolvr
{

template <typename Real, typename Real3, typename CoeffMatrix>
std::pair<Real, Real3> farfield_interaction(const Real & q, const Real3 & xyz, const CoeffMatrix & mu, const Real3 & ref_point, size_t p)
{
    CoeffMatrix mu_shift(1);
    // shift mu from (0,0,0) to (x,y,z)
    L2L(mu, ref_point, mu_shift, xyz, p, 1);

    Real Phi = mu_shift.upper(0, 0).real();
    Real Fx = mu_shift.upper(1, 1).real();
    Real Fy = mu_shift.upper(1, 1).imag();
    Real Fz = mu_shift.upper(1, 0).real();
    Real3 Fxyz = Real3(Fx, Fy, Fz) * q;

    return std::make_pair(Phi, Fxyz);
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
