/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_fmmbase_hpp
#define FMSOLVR_fmmbase_hpp

#include "fmsolvr/abc.hpp"
#include "fmsolvr/multipole.hpp"
#include "fmsolvr/operatorid_m2l.hpp"
#include "fmsolvr/stackedmultireal.hpp"
#include "fmsolvr/tree/simulationbox.hpp"

#include <cstdlib>

namespace fmsolvr {

template <typename Real,
          typename BoxID,
          template <typename> class SIMD_Engine,
          typename Alloc = std::allocator<Real>>
class FMMBaseCoulomb
{
    typedef StackedMultiReal<Real> RealGen;

public:
    typedef Alloc allocator_type;
    typedef std::size_t size_type;
    typedef std::ptrdiff_t index_type;
    typedef BoxID boxid_type;
    typedef M2L_OperatorID<BoxID> operatorid_m2l_type;
    typedef typename RealGen::stacked_real_type real_type;
    typedef typename RealGen::stacked_real3_type real3_type;
    typedef typename RealGen::stacked_real4_type real4_type;
    typedef ABC<real3_type> real33_type;
    typedef SimulationBox<real33_type> simulationbox_type;
#ifdef USE_UPPER
    typedef MultipoleCoefficientsUpper<Real, Alloc> triangular_array_type;
#else
    typedef MultipoleCoefficientsUpperLower<Real, Alloc> triangular_array_type;
#endif
    typedef triangular_array_type multipole_coefficients_type;
    typedef triangular_array_type local_coefficients_type;

    template <typename Tp>
    using SIMDengine = SIMD_Engine<Tp>;
};

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
