/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_handle_tree_hpp
#define FMSOLVR_handle_tree_hpp

#include "fmsolvr/tree/denseoctree.hpp"

namespace fmsolvr {

template <typename FMMTypes>
class FMMTreeHandle
{
    typedef typename FMMTypes::size_type size_type;
    typedef typename FMMTypes::multipole_coefficients_type multipole_coefficients_type;
    typedef typename FMMTypes::local_coefficients_type local_coefficients_type;

    typedef OctreeStorageDense<multipole_coefficients_type> tree_omega_type;
    typedef OctreeStorageDense<local_coefficients_type> tree_mu_type;

private:
    size_type d_;
    size_type p_;

    tree_omega_type * tree_omega_;
    tree_mu_type * tree_mu_;
    multipole_coefficients_type *omega_dipole_compensation_;

public:
    FMMTreeHandle(size_type depth_max, size_type p_max);

    ~FMMTreeHandle();

    size_type depth() const
    { return d_; }

    size_type p() const
    { return p_; }

    const tree_omega_type & tree_omega() const
    { return *tree_omega_; }

    tree_omega_type & tree_omega()
    { return *tree_omega_; }

    const tree_mu_type & tree_mu() const
    { return *tree_mu_; }

    tree_mu_type & tree_mu()
    { return *tree_mu_; }

    multipole_coefficients_type &omega_dp()
    { return *omega_dipole_compensation_; }
};

template <typename FMMTypes>
FMMTreeHandle<FMMTypes>::FMMTreeHandle(size_type depth_max, size_type p_max) :
    d_(depth_max),
    p_(p_max)
{
    tree_omega_ = new tree_omega_type(depth(), p());
    tree_mu_ = new tree_mu_type(depth(), p());
    omega_dipole_compensation_ = new multipole_coefficients_type(p());
}

template <typename FMMTypes>
FMMTreeHandle<FMMTypes>::~FMMTreeHandle()
{
    delete tree_omega_;
    delete tree_mu_;
    delete omega_dipole_compensation_;
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
