/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_ijk_hpp
#define FMSOLVR_ijk_hpp

#include <ostream>

namespace fmsolvr
{

template <typename Integral>
class IJK
{
public:
	typedef Integral value_type;

	IJK(value_type i, value_type j, value_type k): i_(i), j_(j), k_(k)
    { }

    const value_type &i() const
    { return i_; }

    const value_type &j() const
    { return j_; }

    const value_type &k() const
    { return k_; }

private:
	value_type i_, j_, k_;
};

template <typename Integral>
std::ostream &operator<<(std::ostream &os, const IJK<Integral> &ijk)
{
    os << "(" << ijk.i() << ", " << ijk.j() << ", " << ijk.k() << ")";
    return os;
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
