/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_MATH_change_of_basis_hpp
#define FMSOLVR_MATH_change_of_basis_hpp

#include "fmsolvr/math/linear_algebra.hpp"

namespace fmsolvr
{

template <typename Real33>
Real33 change_of_basis_from_standard_basis(const Real33 & abc)
{
	typedef typename Real33::value_type Real3;
	typedef typename Real3::value_type Real;
	// B = (e_1, e_2, e_3), B' = arbitrary basis
	// T^{B}_{B'} = B'^-1 * B = B'^-1
	linear_algebra::SqMatrix<Real> B(3), Binv(3);
	B[0][0] = abc.a().x();
	B[0][1] = abc.a().y();
	B[0][2] = abc.a().z();
	B[1][0] = abc.b().x();
	B[1][1] = abc.b().y();
	B[1][2] = abc.b().z();
	B[2][0] = abc.c().x();
	B[2][1] = abc.c().y();
	B[2][2] = abc.c().z();
	if (!linear_algebra::inverse(3, B, Binv))
		throw "basis matrix not invertible - not a valid basis";
	return Real33(Real3(Binv[0][0], Binv[0][1], Binv[0][2]),
			Real3(Binv[1][0], Binv[1][1], Binv[1][2]),
			Real3(Binv[2][0], Binv[2][1], Binv[2][2]));
}

template <typename Real33>
Real33 change_of_basis_to_standard_basis(const Real33 & abc)
{
	// B = arbitrary basis, B' = (e_1, e_2, e_3)
	// T^{B}_{B'} = B'^-1 * B = B
	return abc;
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
