/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_MATH_ipowlog_hpp
#define FMSOLVR_MATH_ipowlog_hpp

#include <cstddef>

namespace fmsolvr
{

template <typename Integral>
Integral ipow(Integral v, size_t exp)
{
    if (exp == 0 && v == 0)
        return 0;
    Integral res = 1;
    for (size_t i = 0; i < exp; ++i)
        res *= v;
    return res;
}

template <typename Integral>
Integral isquared(Integral v)
{
    return v * v;
}

template <typename Integral>
Integral icubed(Integral v)
{
    return v * v * v;
}

template <typename Integral>
Integral ilog2(Integral v)
{
    Integral res = -1;
    while (v) {
        v >>= 1;
        ++res;
    }
    return res;
}

template <typename Integral>
Integral ilog8(Integral v)
{
    Integral res = -1;
    while (v) {
        v >>= 3;
        ++res;
    }
    return res;
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
