/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_MATH_parity_sign_hpp
#define FMSOLVR_MATH_parity_sign_hpp

#include <cstddef>

namespace fmsolvr
{

template <typename Real>
inline Real powminus1(size_t x)
{
    return (x & 1) ? Real(-1.) : Real(1);
}

template <typename Integral>
inline bool is_odd(Integral x)
{
    return x & 1;
}

// (-1)^k * val
template <typename T, typename Integral>
inline T toggle_sign_if_odd(Integral k, T val)
{
    return is_odd(k) ? -val : val;
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
