/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_MISC_generic_accumulate_hpp
#define FMSOLVR_MISC_generic_accumulate_hpp

#include <functional>

namespace fmsolvr
{

template <typename Storage>
class generic_accumulate
{
    typedef Storage storage_type;
public:
    typedef typename storage_type::value_type value_type;
    typedef typename storage_type::index_type index_type;

    generic_accumulate(storage_type & storage) : storage(storage)
    { }

    void operator () (index_type l, index_type m, const value_type & value)
    {
        storage(l, m) += value;
    }

private:
    storage_type & storage;
};

template <typename StorageT, typename ValueT = StorageT>
class just_accumulate : public std::binary_function<StorageT, ValueT, void>
{
    typedef StorageT storage_type;
public:
    typedef ValueT value_type;

    just_accumulate() { }

    void operator () (storage_type & target, const value_type & value) const
    {
        target += value;
    }
};

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
