/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_OPERATOR_ROTATION_rotation_types_hpp
#define FMSOLVR_OPERATOR_ROTATION_rotation_types_hpp

#include <memory>

#include "fmsolvr/container/linear_array.hpp"
#include "fmsolvr/container/triangular_array.hpp"
#include "fmsolvr/multipole.hpp"
#include "fmsolvr/operator/p3/local2local_p3.hpp"
#include "fmsolvr/operator/p3/multipole2local_p3.hpp"
#include "fmsolvr/operator/p3/multipole2multipole_p3.hpp"
#include "fmsolvr/operator/rotation/eikphitable.hpp"
#include "fmsolvr/operator/rotation/rotation_constants.hpp"
#include "fmsolvr/operator/rotation/rotation_matrix.hpp"
#include "fmsolvr/operator/rotation/rotation_matrix_vector.hpp"
#include "fmsolvr/operator/rotation/rotation_operator.hpp"


//#define USE_UPPER


namespace fmsolvr {

template <typename InputTriangleRotated, typename OutputTriangleRotated, typename ScratchArray>
class RotationScratch
{
public:
    RotationScratch(size_t p) : omuga_source_rot_(p), omuga_target_rot_(p), array_(p)
    { }

    InputTriangleRotated & omuga_source_rot()
    { return omuga_source_rot_; }

    OutputTriangleRotated & omuga_target_rot()
    { return omuga_target_rot_; }

    ScratchArray & array()
    { return array_; }

private:
    InputTriangleRotated omuga_source_rot_;
    OutputTriangleRotated omuga_target_rot_;
    ScratchArray array_;
};


template <typename Real, typename Allocator = std::allocator<Real> >
struct RotationDataTypes
{
    typedef RotationMatrix<Real, Allocator> RotMatrix;
    typedef RotationMatrixVector<RotMatrix, Allocator> RotMatVec;
    typedef EikPhiTable<Real, Allocator> EikPhiTab;
    typedef RotationConstants<Real, Allocator> Rot_Constants;
    typedef RotationOperator<RotMatVec, EikPhiTab, Rot_Constants> Rot_Operator;
    typedef M2Mp3_Operator<Real, Allocator> M2M_Operator;
    typedef M2Lp3_Operator<Real, Allocator> M2L_Operator;
    typedef L2Lp3_Operator<Real, Allocator> L2L_Operator;

    template <typename T>
    struct ScratchTypeGenerator
    {
        typedef UpperTriangularArray<T, void, Allocator> ScratchUpperCoefficientTriangleColumnMajor;
        typedef UpperTriangularArrayRowMajor<T, void, Allocator> ScratchUpperCoefficientTriangleRowMajor;
        typedef LinearArray<T, Allocator> ScratchLinearArray;

        typedef RotationScratch<ScratchUpperCoefficientTriangleRowMajor,
                                ScratchUpperCoefficientTriangleColumnMajor,
                                ScratchLinearArray>
                Scratch;
    };
};

template <typename Real, typename Allocator = std::allocator<Real> >
struct RotationOMuga
{
#ifdef USE_UPPER
    typedef MultipoleCoefficientsUpper<Real, Allocator> CoeffMatrix;
#else
    typedef MultipoleCoefficientsUpperLower<Real, Allocator> CoeffMatrix;
#endif
};

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
