/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_operatorid_m2l_hpp
#define FMSOLVR_operatorid_m2l_hpp

#include "fmsolvr/ijk.hpp"

#include <cstddef>
#include <type_traits>


namespace fmsolvr {

struct DummyBoxID
{
    typedef size_t depth_type;
    typedef ptrdiff_t signed_dimension_type;
};

template <typename BoxID>
class M2L_OperatorID
{
    typedef typename BoxID::depth_type depth_type;
    typedef typename BoxID::signed_dimension_type dimension_type;
    typedef IJK<dimension_type> RelIJK;

    depth_type d_;
    RelIJK ijk_relative_to_source_;

public:
    typedef RelIJK ijk_type;

    template <typename Source, typename Target>
    M2L_OperatorID(const Source &src, const Target &trg, depth_type depth)
            : d_(depth),
              ijk_relative_to_source_(trg.i() - src.i(), trg.j() - src.j(), trg.k() - src.k())
    {
        static_assert(
                std::is_signed<dimension_type>::value,
                "M2L_OperatorID::dimension_type must be signed");
    }

    depth_type d() const
    {
        return d_;
    }

    RelIJK relative_to_source() const
    {
        return ijk_relative_to_source_;
    }

    RelIJK relative_to_target() const
    {
        return RelIJK(
                -ijk_relative_to_source_.i(),
                -ijk_relative_to_source_.j(),
                -ijk_relative_to_source_.k());
    }
};

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
