/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_P2P_base_hpp
#define FMSOLVR_P2P_base_hpp

#include "fmsolvr/stackedmultireal.hpp"


namespace fmsolvr {
namespace pairwise {

template <typename Real, template <typename StackedReal> class Krnl, typename Result, typename Target, typename Source>
class Base
{
    typedef StackedMultiReal<Real> RealGen;

public:
    typedef size_t size_type;
    typedef typename RealGen::stacked_real_type real_type;
    typedef typename RealGen::stacked_real3_type real3_type;
    typedef typename RealGen::stacked_real4_type real4_type;
    typedef Result result_container;
    typedef Target target_container;
    typedef Source source_container;

    Base(Result &result, const Target &target, const Source &source)
        : result_(result), target_(target), source_(source)
    { }

    Result &result()
    { return result_; }

    const Target &target() const
    { return target_; }

    const Source &source() const
    { return source_; }

    template <typename StackedReal>
    static Krnl<StackedReal> Kernel()
    {
        return Krnl<StackedReal>();
    }

private:
    Result &result_;
    const Target &target_;
    const Source &source_;
};

}  // namespace pairwise
}  // namespace fmm

#endif
// vim: et:ts=4:sw=4
