/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_P2P_reducer_hpp
#define FMSOLVR_P2P_reducer_hpp

#include <cstddef>

#include "fmsolvr/stackedmultireal.hpp"


namespace fmsolvr {
namespace pairwise {

        template <typename TheReal, typename Result>
        struct reducer
        {
            typedef size_t size_type;
            typedef typename StackedMultiReal<TheReal>::stacked_real_type Real;
            typedef typename StackedMultiReal<TheReal>::stacked_real3_type Real3;

            Result & result;
            size_type i;
            Real potential;
            Real3 field;

            reducer(Result & result, size_type i)
                : result(result), i(i), potential(0.), field(0., 0., 0.)
            { }

            void operator () (Real p, const Real3 & f)
            {
                potential += p;
                field += f;
            }

            ~reducer()
            {
                result.reduce_pf(i, potential, field);
            }
        };

        // no buffering for deterministic accumulation order
        template <typename TheReal, typename Result>
        struct deterministic_reducer
        {
            typedef size_t size_type;
            typedef typename StackedMultiReal<TheReal>::stacked_real_type Real;
            typedef typename StackedMultiReal<TheReal>::stacked_real3_type Real3;

            Result & result;
            size_type i;

            deterministic_reducer(Result & result, size_type i)
                : result(result), i(i)
            { }

            void operator () (Real p, const Real3 & f)
            {
                result.reduce_fp(i, p, f);
            }
        };

}  // namespace pairwise
}  // namespace fmm

#endif
// vim: et:ts=4:sw=4
