/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_P2P_sequential_hpp
#define FMSOLVR_P2P_sequential_hpp

#include <utility>


namespace fmsolvr {
namespace pairwise {

// this is a "tile generator", i.e. it splits the input tile into
// smaller bits that get handed off to a "tile processor"
template <typename TileProcessor>
class sequential
{
    typedef typename TileProcessor::size_type size_type;
    typedef typename TileProcessor::Real Real;
    typedef typename TileProcessor::Real3 Real3;

    TileProcessor tile_processor;

public:
    template <typename... Args>
    sequential(Args&&... args) : tile_processor(std::forward<Args>(args)...)
    { }

            void overlapping_interactions(size_type i_begin, size_type i_end, size_type j_begin, size_type j_end)
            {
                tile_processor.overlapping_interactions(i_begin, 1, i_end, j_begin, j_end);
            }

            void disjoint_interactions(size_type i_begin, size_type i_end, size_type j_begin, size_type j_end)
            {
                tile_processor.disjoint_interactions(i_begin, 1, i_end, j_begin, j_end);
            }

            void offsetted_interactions(size_type i_begin, size_type i_end, size_type j_begin, size_type j_end, Real3 offset)
            {
                tile_processor.offsetted_interactions(i_begin, 1, i_end, j_begin, j_end, offset);
            }

            void single_scaled_interaction(size_type i, size_type j, Real scale, Real3 offset)
            {
                tile_processor.single_scaled_interaction(i, j, scale, offset);
            }
};

}  // namespace pairwise
}  // namespace fmm

#endif
// vim: et:ts=4:sw=4
