/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_SIMD_Vec_m512_hpp
#define FMSOLVR_SIMD_Vec_m512_hpp

// __m512 = float[16] (AVX512)

#include "fmsolvr/simd/Vec_config.hpp"

#ifdef FMSOLVR_HAVE_AVX512F
#if !(defined __ICC) || (__ICC >= 1800)

#include "fmsolvr/simd/Vec_base.hpp"
#include <immintrin.h>

namespace fmsolvr
{

template <>
struct Vec_traits<__m512> : public Vec_base<__m512, float>
{
	typedef Vec_base<__m512, float> base;
	typedef typename base::vec_type vec_type;
	typedef typename base::scalar_type scalar_type;

	vec_type zero() const
	{
		vec_type z = {0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.};
		return z;
	}

	vec_type same(scalar_type s) const
	{
		vec_type v = {s, s, s, s, s, s, s, s, s, s, s, s, s, s, s, s};
		return v;
	}

	scalar_type sum(const vec_type & v) const
	{
		const scalar_type * p = reinterpret_cast<const scalar_type *>(&v);
		scalar_type s = 0.;
		for (size_t i = 0; i < VecSize; ++i)
			s += p[i];
		return s;
	}
};

template <>
inline
__m512 rsqrt_exact(__m512 a)
{
	__m512 sqr = _mm512_sqrt_ps(a);
	__m512 ones = {1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.};
	return ones / sqr;
}

}  // namespace fmsolvr

#endif  // !__ICC
#endif  // FMSOLVR_HAVE_AVX512F

#endif
