/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_SIMD_Vec_wrap_m512d_hpp
#define FMSOLVR_SIMD_Vec_wrap_m512d_hpp

// __m512d = double[8] (AVX512)

#include "fmsolvr/simd/Vec_config.hpp"

#ifdef FMSOLVR_HAVE_AVX512F

#include "fmsolvr/simd/Vec_base.hpp"
#include "fmsolvr/simd/wrap_m512d.hpp"
#include <immintrin.h>

namespace fmsolvr
{

template <>
struct Vec_traits<wrap_m512d> : public Vec_base<wrap_m512d, double>
{
	typedef Vec_base<wrap_m512d, double> base;
	typedef typename base::vec_type vec_type;
	typedef typename base::scalar_type scalar_type;

	vec_type zero() const
	{
		__m512d z = {0., 0., 0., 0., 0., 0., 0., 0.};
		return z;
	}

	vec_type same(scalar_type s) const
	{
		__m512d v = _mm512_broadcastsd_pd(_mm_load_sd(&s));
		return v;
	}

	scalar_type sum(const vec_type & v) const
	{
		const scalar_type * p = reinterpret_cast<const scalar_type *>(&v);
		scalar_type s = 0.;
		for (size_t i = 0; i < VecSize; ++i)
			s += p[i];
		return s;
	}
};

template <>
inline
wrap_m512d rsqrt_exact(wrap_m512d a)
{
	wrap_m512d sqr = _mm512_sqrt_pd(a);
	__m512d ones_ = {1., 1., 1., 1., 1., 1., 1., 1.};
	wrap_m512d ones = ones_;
	return ones / sqr;
}

}  // namespace fmsolvr

#endif  // FMSOLVR_HAVE_AVX512F

#endif
