/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_SIMD_wrap_m512_hpp
#define FMSOLVR_SIMD_wrap_m512_hpp

#include "fmsolvr/simd/Vec_config.hpp"

#ifdef FMSOLVR_HAVE_AVX512F

#include <immintrin.h>

namespace fmsolvr
{

class wrap_m512
{
    __m512 val_;

public:
    static wrap_m512 undefined()
    {
        return _mm512_undefined_ps();
    }

    static wrap_m512 zero()
    {
        return _mm512_setzero_ps();
    }

    wrap_m512()
    { val_ = undefined(); }

    wrap_m512(const __m512 &a)
    { val_ = a; }

    operator __m512() const
    { return val_; }

    wrap_m512 operator -() const
    {
        return _mm512_sub_ps(zero(), val_);
    }

    wrap_m512 operator +(const wrap_m512 &a) const
    {
        return _mm512_add_ps(val_, a);
    }

    wrap_m512 operator -(const wrap_m512 &a) const
    {
        return _mm512_sub_ps(val_, a);
    }

    wrap_m512 operator *(const wrap_m512 &a) const
    {
        return _mm512_mul_ps(val_, a);
    }

    wrap_m512 operator /(const wrap_m512 &a) const
    {
        return _mm512_div_ps(val_, a);
    }

    wrap_m512& operator +=(const wrap_m512 &a)
    {
        val_ = _mm512_add_ps(val_, a);
        return *this;
    }

    wrap_m512& operator -=(const wrap_m512 &a)
    {
        val_ = _mm512_sub_ps(val_, a);
        return *this;
    }

    wrap_m512& operator *=(const wrap_m512 &a)
    {
        val_ = _mm512_mul_ps(val_, a);
        return *this;
    }
};

}  // namespace fmsolvr

#endif  // FMSOLVR_HAVE_AVX512F

#endif
// vim: et:ts=4:sw=4
