/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_sourcecounters_hpp
#define FMSOLVR_sourcecounters_hpp

#include <cstddef>


namespace fmsolvr {

// counts the number of source particles (and maybe dipoles, ... in the future)
// that found their way into a multipole expansion (via P2M, M2M) and later
// into a local expansion (via M2L and L2L)
class SourceCounters {
    typedef size_t size_type;

    size_type particles_;

public:
    SourceCounters()
    {
        clear();
    }

    void clear()
    {
        particles_ = 0;
    }

    void inc()
    {
        ++particles_;
    }

    void add(const SourceCounters & a)
    {
        particles_ += a.particles_;
    }

    bool empty() const
    {
        return particles_ == 0;
    }

    size_type particles() const
    { return particles_; }
};

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
