#ifndef _AnBe_FMM_pass12345_hpp_
#define _AnBe_FMM_pass12345_hpp_

#include "fmsolvr/fmm_enum.hpp"
#if 0
#include "fmsolvr/sequential/pass1.hpp"
#include "fmsolvr/sequential/pass2.hpp"
#include "fmsolvr/sequential/pass2dl.hpp"
#include "fmsolvr/sequential/pass3.hpp"
#include "fmsolvr/sequential/pass4.hpp"
#include "fmsolvr/sequential/pass5.hpp"
#else
#include "fmsolvr/tasks/pass1.hpp"
#include "fmsolvr/tasks/pass2.hpp"
#include "fmsolvr/sequential/pass2dl.hpp"
#include "fmsolvr/tasks/pass3.hpp"
#include "fmsolvr/tasks/pass4.hpp"
#include "fmsolvr/tasks/pass5.hpp"
#endif
#include "fmsolvr/util/TimeLine.hpp"
#include "fmsolvr/util/checking_helpers.hpp"
#include "fmsolvr/util/envtools.hpp"
#include "fmsolvr/util/wrpapi.hpp"

#include "TreePartition.hpp"
#include "parallelization/LoadBalancerHelper.hpp"

namespace fmsolvr {

namespace {

template <typename T>
void unused(const T &)
{ }

}  // anonymous namespace

template <typename FMMHandle, typename IntraNodeContainer>
void pass12345(papiwrapper<> &pw, FMMHandle & fmm_handle, IntraNodeContainer &intra_node_container)
{
    auto & task_factory = intra_node_container.task_factory();
    auto &verbout = fmm_handle.verbout();

    auto & particle_handle = fmm_handle.particle_handle();
    const auto & simulationbox = particle_handle.simulationbox();

#if 0
    // regular passes
    const bool doCompensateDipole = !simulationbox.open_boundary_conditions() &&
                                    (get_env_int("DIPOLE_COMPENSATION", 1) != 0);
    const bool doPass1P2M = (get_env_int("PASS1P2M", 1) != 0);
    const bool doPass1M2M = (get_env_int("PASS1M2M", 1) != 0);
    const bool doPass2 = (get_env_int("PASS2", 1) != 0);
    const bool doPass2Lattice = (get_env_int("PASS2LATTICE", 1) != 0);
    const bool doPass3 = (get_env_int("PASS3", 1) != 0);
    const bool doPass4 = (get_env_int("PASS4", 1) != 0);
    const bool doPass5 = (get_env_int("PASS5", 1) != 0);
#endif
    const bool doCheckInteractionCount = (get_env_int("CHECKINTERACTIONCOUNT", 1) != 0);

    typedef typename FMMHandle::size_type size_type;
    typedef typename FMMHandle::boxid_type BoxID;

#if 0
    typedef typename FMMHandle::P2P_type P2P_type;
#endif

    //typedef timeline::TimeLineNone<FMMActivity> TimeLine;
    typedef timeline::TimeLinePapiPrint<FMMActivity> TimeLine;
    //typedef timeline::TimeLineAccumulated<FMMActivity> TimeLine;
    //typedef timeline::TimeLineDetailed<FMMActivity> TimeLine;

    const size_type n = particle_handle.n_active();
    const size_type ws = fmm_handle.ws();

    TimeLine timeline(fmm_handle.verbose() ? &pw : NULL);

#if 0

    // verbout << "# Features:";
    // verbout << " Pass1P2M:" << doPass1P2M;
    // verbout << " Pass1M2M:" << doPass1M2M;
    // verbout << " Pass2:" << doPass2;
    // verbout << " Pass2Lattice:" << doPass2Lattice;
    // verbout << " Pass3:" << doPass3;
    // verbout << " Pass4:" << doPass4;
    // verbout << " Pass5:" << doPass5;
    // verbout << " DipoleCompensation:" << doCompensateDipole;
    // verbout << std::endl;


    // *** near field ***

    // PASS 5: P2P
    if (doPass5)
    {
        verbout << "# PASS 5: P2P\n";
        auto activity = timeline.startActivity(FMMActivity::P2P);
        size_t p2pcount = 0;
        auto particles = particle_handle.particle_adapter();
        auto result = particle_handle.potential_efield_nf_adapter();

        P2P_type P2P(result, particles, particles);
        auto pass5result = pass5(fmm_handle, P2P);
        p2pcount += pass5result.first;
        particle_handle.add_interactions_nf(pass5result.second);
        timeline.stopActivity(activity, pass5result.second);
        pw.report(verbout, p2pcount);  // alternative normalization
    }

    // *** far field ***

    // PASS 1: P2M
    if (doPass1P2M)
    {
        verbout << "# PASS 1: P2M\n";
        auto activity = timeline.startActivity(FMMActivity::P2M);
        pass1_P2M(fmm_handle);
        timeline.stopActivity(activity, n);
    }

    // PASS 1: M2M
    if (doPass1M2M)
    {
        verbout << "# PASS 1: M2M\n";
        auto activity = timeline.startActivity(FMMActivity::M2M);
        size_t m2mcount = 0;
        m2mcount += pass1_M2M_tree(fmm_handle);
        timeline.stopActivity(activity, m2mcount);
    }

    assert(checking::is_finite<BoxID>(fmm_handle.tree_omega(), depth, p));

    // tree_omega has had populate_lower called for all elements, the omegas are ready to use

    // PASS 2: M2L
    if (doPass2)
    {
        double t_pass2 = -timestamp();
        //all M2L calls are independent: for each target box all contributors determined and combined into target box
        //with WS=1 each target box has 189 contributors
        verbout << "# PASS 2: M2L\n";
        for (size_t d = 1; d <= depth; ++d) {
            auto activity = timeline.startActivity(FMMActivity::M2L);
            size_t m2lcount = 0;
            m2lcount += pass2_M2L_one_level(fmm_handle, d);
            timeline.stopActivity(activity, m2lcount);
        }
        t_pass2 += timestamp();
        verbout << "Pass 2 M2L elapsed: " << t_pass2 << std::endl;
    }

    // PASS 2: Lattice (+ Dipole Compensation)
    if (doPass2Lattice)
    {
        if (!simulationbox.open_boundary_conditions())
        {
            verbout << "# PASS 2: Lattice\n";
            auto activity = timeline.startActivity(FMMActivity::Lattice);
            pass2_dipole_lattice(fmm_handle);
            timeline.stopActivity(activity, 1);
        }
    }

    // tree_mu needs populate_lower() to be called for all elements before being used
    // only tree_mu[root = BoxID(0, 0, 0, 0)] is complete at this point, L2L will accumulate more into the deeper levels

    assert(checking::is_finite<BoxID>(fmm_handle.tree_mu(), depth, p));

    // PASS 3: L2L
    if (doPass3)
    {
        verbout << "# PASS 3: L2L\n";
        auto activity = timeline.startActivity(FMMActivity::L2L);
        size_t l2lcount = 0;
        l2lcount += pass3_L2L_tree(fmm_handle);
        timeline.stopActivity(activity, l2lcount);
    }

    assert(checking::is_finite<BoxID>(fmm_handle.tree_mu(), depth, p));

    // tree_mu has had populate_lower called for all elements, the are ready to use

    // PASS 4: Potential and Field
    if (doPass4)
    {
        verbout << "# PASS 4: phi & f\n";
        auto activity = timeline.startActivity(FMMActivity::L2P);
        pass4(fmm_handle);
        timeline.stopActivity(activity, n);
    }
#endif
    auto & tree_omega = fmm_handle.tree_omega();
    auto & tree_mu = fmm_handle.tree_mu();
    fmm_handle.particle_handle().zero();
    for (unsigned d = 0; d <= fmm_handle.depth(); ++d)
    {
        for (auto &mu : tree_mu.LevelRange(d))
        {
            mu.zero(fmm_handle.p());
            mu.clearSourceCounters();
        }
        for (auto &omega : tree_omega.LevelRange(d))
        {
            omega.zero(fmm_handle.p());
            omega.clearSourceCounters();
        }
    }

    size_t d = fmm_handle.depth();
    const load_balancer::details::EqualDist ed(
            fmm_handle.particle_handle().num_boxes_lowest(), intra_node_container.num_threads());
    auto bs_vec = ed.GetBSVector();
    typename BoxID::sfcindex_type start_id = 0;

    for (auto bs : bs_vec)
    {
        typename FMMHandle::tree_partition_type tp(start_id, start_id + bs, d);
        task_factory.CreateAndEnqueueTask(task_factory.p2m, tp);
        start_id += bs;
    }

    for (unsigned i = 0; i < fmm_handle.particle_handle().num_boxes_lowest(); ++i)
        task_factory.CreateAndEnqueueTask(
                task_factory.p2p, typename FMMHandle::tree_partition_type(i, d));
    intra_node_container.EnableWorkStealing();
    intra_node_container.FinishAndSoftJoin();

    size_t nf_interactions = particle_handle.interactions_nf();
    size_t ff_interactions = particle_handle.interactions_ff();

    verbout << "# interactions: n=" << n << " nf=" << nf_interactions << " ff=" << ff_interactions << std::endl;
    if (doCheckInteractionCount)
    {
        if (simulationbox.open_boundary_conditions())
            assert(nf_interactions + ff_interactions == n * (n - 1));
        else
            assert(nf_interactions + ff_interactions ==
                   n * (n - 1) + n * n * ((2 * ws + 1) * (2 * ws + 1) * (2 * ws + 1) - 1));
    }
#ifdef NDEBUG
    unused(BoxID(0, 0, 0, 0));
    unused(ws);
#endif
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
