#ifndef _AnBe_FMM_pass4_hpp_
#define _AnBe_FMM_pass4_hpp_

#include "fmsolvr/operator/p4/local2local.hpp"
#include "fmsolvr/operator/p4/local2local_derivative.hpp"


namespace fmsolvr {
template <typename FMMHandle, typename MutexContainer, typename DependencyManager>
void pass4_one_box(FMMHandle &fmm_handle, size_t id, MutexContainer &mutex_container, DependencyManager & dm)
{
    typedef typename FMMHandle::real3_type Real3;
    typedef typename FMMHandle::real_type Real;
    typedef typename FMMHandle::size_type size_type;
    typedef typename FMMHandle::boxid_type BoxID;
    typedef typename FMMHandle::multipole_coefficients_type multipole_coefficients_type;
    typedef typename FMMHandle::local_coefficients_type local_coefficients_type;
    typedef typename local_coefficients_type::complex_type complex_type;

    auto & particle_handle = fmm_handle.particle_handle();
    const auto & tree_mu = fmm_handle.tree_mu();
    const size_type depth = fmm_handle.depth();
    const size_type p = fmm_handle.p();
    const auto & div_lut_p = fmm_handle.div_lut_p(p);
    const auto & ordered_particles = particle_handle.ordered_particles();
    const auto & box_particle_offset = particle_handle.box_particle_offset();
    auto result = particle_handle.potential_efield_ff_adapter();

    size_t ff_interactions = 0;
    //This is parallel for all particles
    //-> one thread per paticle each particle reads the taylor moment generated in PASS 1-3 of its box
    //the taylor moment will likely fit into shared memory
    local_coefficients_type dmux(p);
    local_coefficients_type dmuy(p);
    local_coefficients_type dmuz(p);
    multipole_coefficients_type omega(p);
    {
        if (box_particle_offset[id] == box_particle_offset[id + 1])
        {
            // no target particles in this box
            return;
        }

        BoxID bid(id, depth);

        const auto & mu = tree_mu[bid];
        const size_t source_particles = mu.sourceCounters().particles();
        Real3 expansion_point_mu_ = particle_handle.simulationbox().expansion_point(bid);

        DxL2L_reference(mu, dmux, p);
        DyL2L_reference(mu, dmuy, p);
        DzL2L_reference(mu, dmuz, p);
        dmux.populate_lower();
        dmuy.populate_lower();
        dmuz.populate_lower();

        for (size_t i = box_particle_offset[id]; i < box_particle_offset[id + 1]; ++i)
        {
            Real3 xyz = Real3(ordered_particles[i]);
            Real q = -1.;  // Field computation only

            // far field Field
            // FIXME: excluding the sign, omega and O should be the same
            omega.zero(p);
            P2M(q, xyz, omega, expansion_point_mu_, div_lut_p);
            omega.populate_lower();

            // far field Potential
            L2L_Operator<local_coefficients_type> O(expansion_point_mu_, xyz, p);

            // FIXME: optimize multiplication: (re, 0) = (re, im) * (re, im)
            complex_type Phi_i(0.);
            complex_type Fx(0.);
            complex_type Fy(0.);
            complex_type Fz(0.);

            for (ssize_t l = 0; l <= ssize_t(p); ++l)
            {
                for (ssize_t m = -l; m <= l; ++m)
                {
                    Phi_i += mu.get(l, m) * O.get(l, m);
                    complex_type omega_lm = omega.get(l, m);
                    Fx += dmux.get(l, m) * omega_lm;
                    Fy += dmuy.get(l, m) * omega_lm;
                    Fz += dmuz.get(l, m) * omega_lm;
                }
            }
            auto &mutex = mutex_container.lowest_lvl_particle_mutexes_[id];
            mutex.lock();
            // FIXME: verify imag() part is (reasonably) zero
            result.reduce_pf(i, Phi_i.real(), -Real3(Fx.real(), Fy.real(), Fz.real()));
            mutex.unlock();
            ff_interactions += source_particles;
        }
    }
    particle_handle.add_interactions_ff(ff_interactions);
    dm.dc().force_computation_counter -= (box_particle_offset[id + 1] - box_particle_offset[id]);
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
