/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_TREE_boxexpansionpoint_hpp
#define FMSOLVR_TREE_boxexpansionpoint_hpp

#include "fmsolvr/math/reciprocal.hpp"

namespace fmsolvr {

// compute expansion point for a source at zero-based sub-box-coordinates (i, j, k), origin at (.5, .5, .5)
// the result is relative to a target at origin (0., 0., 0.)
// 0 <= i,j,k < dim <= (1 << depth)
template <typename Real33>
typename Real33::value_type box_expansion_point(const Real33 & abc, size_t depth, std::ptrdiff_t dim, std::ptrdiff_t i, std::ptrdiff_t j, std::ptrdiff_t k)
{
    typedef typename Real33::value_type Real3;
    typedef typename Real3::value_type Real;

    Real3 expn_point = Real3(-(dim - 1) + 2 * i, -(dim - 1) + 2 * j, -(dim - 1) + 2 * k) * reciprocal<Real>(2 * (size_t(1) << depth));
    return abc * expn_point;
}

template <typename Real33>
typename Real33::value_type relative_box_expansion_point(const Real33 & abc, size_t depth, std::ptrdiff_t i, std::ptrdiff_t j, std::ptrdiff_t k)
{
    typedef typename Real33::value_type Real3;
    typedef typename Real3::value_type Real;

    Real3 expn_point = Real3(i, j, k) * reciprocal<Real>(size_t(1) << depth);
    return abc * expn_point;
}

} // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
