/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_TREE_boxid_hpp
#define FMSOLVR_TREE_boxid_hpp

namespace fmsolvr {

// To implement the BoxID interface a class should provide the following:
//
// Types:
//   depth_type (optional?)
//   dimension_type
//   sfcindex_type
//
// Constructors:
//   BoxID(dimension_type i, dimension_type j, dimension_type k, depth_type d)
//   BoxID4D(sfcindex_type sfcidx, depth_type d)
//
// Methods:
//   static constexpr size_t max_depth()
//   dimension_type i() const
//   dimension_type j() const
//   dimension_type k() const
//   depth_type d() const
//   sfcindex_type sfcindex() const
//
//   Self parent() const
//   Self child(dimension_type i, dimension_type j, dimension_type k) const
//   Self descendant(dimension_type i, dimension_type j, dimension_type k,
//                   depth_type delta_depth) const
//

template <typename BoxIdImpl>
class BoxID : public BoxIdImpl
{
public:
    template <typename... Args>
    BoxID(Args... args) : BoxIdImpl(args...)
    { }
};

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
