/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_UTIL_dump_helpers_hpp
#define FMSOLVR_UTIL_dump_helpers_hpp

#include <iostream>

namespace fmsolvr
{

template <typename LinearArray>
void dump_list_array(const LinearArray &a, size_t p)
{
    std::cout << "# k\tval" << std::endl;
    for (size_t k = 0; k <= p; ++k)
    {
        std::cout << k << '\t' << a[k] << std::endl;
    }
}


template <typename LinearArray>
void dump_list_array_complex(const LinearArray &a, size_t p)
{
    std::cout << "# k\tre\tim" << std::endl;
    for (size_t k = 0; k <= p; ++k)
    {
        std::cout << k << '\t' << a[k].real() << '\t' << a[k].imag() << std::endl;
    }
}


template <typename TriangularArray>
void dump_list_upper_triangle_complex(const TriangularArray &a, size_t p)
{
    std::cout << "# l\tm\tre\tim" << std::endl;
    for (size_t l = 0; l <= p; ++l)
    {
        for (size_t m = 0; m <= l; ++m)
        {
            std::cout << l << '\t' << m << '\t' << a.upper(l, m).real() << '\t'
                      << a.upper(l, m).imag() << std::endl;
        }
    }
}


template <typename QuarterPyramid>
void dump_list_quarter_pyramid_real2(const QuarterPyramid &a, size_t p)
{
    std::cout << "# l\tm\tk\tval[0]\tval[1]" << std::endl;
    for (size_t l = 0; l <= p; ++l)
    {
        for (size_t m = 0; m <= l; ++m)
        {
            for (size_t k = 0; k <= l; ++k)
            {
                const auto &val = a(l, m, k);
                std::cout << l << '\t' << m << '\t' << k << '\t' << val[0] << '\t' << val[1]
                          << std::endl;
            }
        }
    }
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
