/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_UTIL_wrpapi_hpp
#define FMSOLVR_UTIL_wrpapi_hpp

#include <ctime>
#include <iostream>
#include <sys/time.h>

namespace fmsolvr
{

// Returns number of seconds since the epoch, high resolution.
inline double
timestamp()
{
    struct timeval tp;
    gettimeofday(&tp, NULL);
    return double(tp.tv_sec) + tp.tv_usec / 1000000.;
}

}  // namespace fmsolvr

#ifdef WITH_PAPI
// there may be a mismatch with the ffsll() prototype in papi.h
// if string.h is included later
#include <cstring>

extern "C" {
#include <papi.h>
}

namespace fmsolvr
{

template <typename = void>
class papiwrapper
{
	static const int max_events = 10;
	int event[max_events];
	long long values[max_events];
	int num_events;

	int EventSet;
	int start_result;
	int stop_result;
	double start_time;
	double stop_time;

public:
	papiwrapper();
	~papiwrapper();
	void start()
	{
		start_time = timestamp();
		start_result = PAPI_start(EventSet);
	}
	void stop()
	{
		stop_result = PAPI_stop(EventSet, values);
		stop_time = timestamp();
	}

    void report(std::ostream &os, double, double = 0, double = 0);

    // FIXME: deprecate this
    void report(double steps, double bytes_load = 0, double bytes_store = 0)
    {
        report(std::cout, steps, bytes_load, bytes_store);
    }
};

}  // namespace fmsolvr

#else

namespace fmsolvr
{

template <typename = void>
struct papiwrapper
{
	double start_time;
	double stop_time;

	papiwrapper() { }
	void start()
	{
		start_time = timestamp();
	}
	void stop()
	{
		stop_time = timestamp();
	}

    void report(std::ostream &os, double, double = 0, double = 0);

    // FIXME: deprecate this
    void report(double steps, double bytes_load = 0, double bytes_store = 0)
    {
        report(std::cout, steps, bytes_load, bytes_store);
    }
};

}  // namespace fmsolvr

#endif

#include "fmsolvr/util/wrpapi_impl.hpp"

#endif
// vim: et:ts=4:sw=4
