/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef FMSOLVR_xyzssf_hpp
#define FMSOLVR_xyzssf_hpp

#include "fmsolvr/xyzs.hpp"

namespace fmsolvr
{

// coordinates x, y, z and strength s (e.g. charge q or mass m)
// as well as site and form
template <typename Real, typename Int>
class XYZSSF : public XYZS<Real>
{
public:
    using typename XYZS<Real>::value_type;
    typedef Int index_type;

    XYZSSF() : XYZS<Real>(), site_(0), form_(0)
    { }

    XYZSSF(value_type x, value_type y, value_type z, value_type s, index_type site, index_type form)
            : XYZS<Real>(x, y, z, s), site_(site), form_(form)
    { }

    const index_type &site() const
    { return site_; }

    const index_type &form() const
    { return form_; }

private:
    index_type site_, form_;
};

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
