/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <iostream>

#include "linear_algebra.hpp"

template <typename Matrix, typename Vector>
void dump(size_t dim, const Matrix & A, const Vector & x, const Vector & b)
{
    for (size_t i = 0; i < dim; ++i) {
        std::cout << "( ";
        for (size_t j = 0; j < dim; ++j) {
            std::cout << A[i][j] << "\t";
        }
        std::cout << ") ( x" << i << " )  =  ( " << b[i] << "\t)" << std::endl;
    }
    for (size_t i = 0; i < dim; ++i)
        std::cout << "x" << i << "=" << x[i] << ", ";
    std::cout << std::endl;
}

int main()
{
    typedef double Real;
    {
        const size_t dim = 2;
        linear_algebra::SqMatrix<Real> A(dim);
        A[0][0] = 1;
        A[0][1] = 2;
        A[1][0] = 3;
        A[1][1] = 4;
        Real x[dim];
        Real b[dim] = {5, 6};

        linear_algebra::solve(dim, A, x, b);

        dump(dim, A, x, b);
    }
    {
        const size_t dim = 3;
        linear_algebra::SqMatrix<Real> A(dim);
        A[0][0] = 1;
        A[0][1] = 2;
        A[0][2] = 5;
        A[1][0] = 3;
        A[1][1] = 4;
        A[1][2] = 6;
        A[2][0] = 7;
        A[2][1] = 8;
        A[2][2] = 9;
        Real x[dim];
        Real b[dim] = {12, 11, 10};

        linear_algebra::solve(dim, A, x, b);

        dump(dim, A, x, b);
    }
    {
        const size_t dim = 4;
        linear_algebra::SqMatrix<Real> A(dim);
        A[0][0] = 1;
        A[0][1] = 2;
        A[0][2] = 5;
        A[0][3] = 6;
        A[1][0] = 3;
        A[1][1] = 4;
        A[1][2] = 6;
        A[1][3] = 7;
        A[2][0] = 7;
        A[2][1] = 8;
        A[2][2] = 9;
        A[2][3] = 8;
        A[3][0] = 5;
        A[3][1] = 3;
        A[3][2] = 7;
        A[3][3] = 2;
        Real x[dim];
        Real b[dim] = {12, 11, 10, 9};

        linear_algebra::solve(dim, A, x, b);

        dump(dim, A, x, b);
    }

    return 0;
}

// vim: et:ts=4:sw=4
