/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <iostream>
#include <chrono>

#include "wrpapi.hpp"


papiwrapper pw;

typedef std::chrono::high_resolution_clock hrclk;

void bench(int n)
{
    auto before = hrclk::now();
    for (int i = 0; i < n; ++i)
    {
        pw.start();
        pw.stop();
    }
    auto after = hrclk::now();
    auto elapsed1 = after - before;

    auto ns1 = std::chrono::nanoseconds(elapsed1).count();

    std::cout << n << "\t" << double(ns1) / double(n) << std::endl;
}

int main()
{
    std::cout << "# n\tt" << std::endl;
    bench(1000);
    bench(10000);
    bench(100000);
    bench(1000000);
    bench(10000000);
    bench(100000000);
    bench(1000000000);
}
