/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <iostream>
#include <vector>
#include "wrpapi.hpp"

int main(int argc, char **argv)
{
    typedef uint64_t Type;

    if (argc < 2)
    {
        std::cout << "Usage: " << argv[0] << " num_elements [ops] [max_stride]" << std::endl;
        return 1;
    }

    size_t sz = atol(argv[1]);
    std::vector<Type> vec(sz);

    size_t ops = 1000000;
    if (argc > 2)
        ops = atol(argv[2]);

    size_t max_stride = (vec.size() + 1) / 2;
    if (argc > 3)
        max_stride = std::min<size_t>(max_stride, atol(argv[3]));

    std::cout << "# stride time_ns/op (sizeof(Type)=" << sizeof(Type) << ")" << std::endl;
    for (size_t stride = 0; stride < max_stride; ++stride)
    {
        double t = -timestamp();
        for (size_t i = 0, idx = 0; i < ops; ++i)
        {
            ++vec[idx];
            idx += stride;
            if (idx >= vec.size())
                idx = 0;  // reset to 0, not %vec.size()
        }
        t += timestamp();
        std::cout << stride << "\t" << t * 1E9 / double(ops) << std::endl;
    }
}
// vim: et:ts=4:sw=4
