/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#ifndef _AnBe_FMM_MICROBENCH_complex_mul_acc_hpp_
#define _AnBe_FMM_MICROBENCH_complex_mul_acc_hpp_

#include "Vec.h"


template <typename SIMDT, typename ScalarT>
inline
__attribute__((noinline))
void split_complex_mul_acc(
        SIMDT & are,
        SIMDT & aim,
        const ScalarT * pra,
        const ScalarT * pia,
        const ScalarT * prb,
        const ScalarT * pib,
        size_t n)
{
    typedef SIMDT simd_type;
    Vec_traits<simd_type> VT;
    const size_t s = VT.VecSize;

    n -= n % (3 * s); // HACK! only process stuff divisable by 3
    if (n == 0)
        return;

    register simd_type arr = VT.zero();
    register simd_type aii = VT.zero();
    register simd_type ari = VT.zero();
    register simd_type air = VT.zero();

    register simd_type y0;
    register simd_type y1;
    register simd_type y2;
    register simd_type y3;
    register simd_type y4;
    register simd_type y5;
    register simd_type y6;
    register simd_type y7;
    register simd_type y8;
    register simd_type y9;
    register simd_type yA;
    register simd_type yB;

    size_t i = 0;
    y0 = VT.unaligned_load(pra + i + 0 * s);
    y1 = VT.unaligned_load(prb + i + 0 * s);
    y2 = VT.unaligned_load(pib + i + 0 * s);
    y3 = VT.unaligned_load(pia + i + 0 * s);
    y4 = VT.unaligned_load(pra + i + 1 * s);
    y5 = VT.unaligned_load(prb + i + 1 * s);              yB = y0 * y1;
    y6 = VT.unaligned_load(pib + i + 1 * s);              y0 = y0 * y2;
    y7 = VT.unaligned_load(pia + i + 1 * s);              y1 = y1 * y3;
    y8 = VT.unaligned_load(pra + i + 2 * s);              y2 = y2 * y3;
    y9 = VT.unaligned_load(prb + i + 2 * s);              y3 = y4 * y5;
    yA = VT.unaligned_load(pib + i + 2 * s);  arr += yB;  y4 = y4 * y6;
    yB = VT.unaligned_load(pia + i + 2 * s);  ari += y0;  y5 = y5 * y7;
    i += 3 * s;
    for ( ; i < n; i += 3 * s) {
        y0 = VT.unaligned_load(pra + i + 0 * s);  air += y1;  y6 = y6 * y7;
        y1 = VT.unaligned_load(prb + i + 0 * s);  aii += y2;  y7 = y8 * y9;
        y2 = VT.unaligned_load(pib + i + 0 * s);  arr += y3;  y8 = y8 * yA;
        y3 = VT.unaligned_load(pia + i + 0 * s);  ari += y4;  y9 = y9 * yB;
        y4 = VT.unaligned_load(pra + i + 1 * s);  air += y5;  yA = yA * yB;
        y5 = VT.unaligned_load(prb + i + 1 * s);  aii += y6;  yB = y0 * y1;
        y6 = VT.unaligned_load(pib + i + 1 * s);  arr += y7;  y0 = y0 * y2;
        y7 = VT.unaligned_load(pia + i + 1 * s);  ari += y8;  y1 = y1 * y3;
        y8 = VT.unaligned_load(pra + i + 2 * s);  air += y9;  y2 = y2 * y3;
        y9 = VT.unaligned_load(prb + i + 2 * s);  aii += yA;  y3 = y4 * y5;
        yA = VT.unaligned_load(pib + i + 2 * s);  arr += yB;  y4 = y4 * y6;
        yB = VT.unaligned_load(pia + i + 2 * s);  ari += y0;  y5 = y5 * y7;
    }
    air += y1;  y6 = y6 * y7;
    aii += y2;  y7 = y8 * y9;
    arr += y3;  y8 = y8 * yA;
    ari += y4;  y9 = y9 * yB;
    air += y5;  yA = yA * yB;
    aii += y6;
    arr += y7;
    ari += y8;
    air += y9;
    aii += yA;

    are += arr - aii;
    aim += ari + air;
}

#if 0
".align 16\n"
"       vaddpd  %%ymm0, %%ymm0,  %%ymm0\n"
"       vaddpd  %%ymm15,%%ymm0,  %%ymm0\n"
"       vaddpd  %%ymm0, %%ymm15, %%ymm0\n"
"       vaddpd  %%ymm15,%%ymm15, %%ymm0\n"
"       vaddpd  %%ymm0, %%ymm0,  %%ymm15\n"
"       vaddpd  %%ymm15,%%ymm0,  %%ymm15\n"
"       vaddpd  %%ymm0, %%ymm15, %%ymm15\n"
"       vaddpd  %%ymm15,%%ymm15, %%ymm15\n"
"       vmulpd  %%ymm0, %%ymm0,  %%ymm0\n"
"       vmulpd  %%ymm15,%%ymm0,  %%ymm0\n"
"       vmulpd  %%ymm0, %%ymm15, %%ymm0\n"
"       vmulpd  %%ymm15,%%ymm15, %%ymm0\n"
"       vmulpd  %%ymm0, %%ymm0,  %%ymm15\n"
"       vmulpd  %%ymm15,%%ymm0,  %%ymm15\n"
"       vmulpd  %%ymm0, %%ymm15, %%ymm15\n"
"       vmulpd  %%ymm15,%%ymm15, %%ymm15\n"
"       vmovupd (%%rcx,%%rax,1), %%ymm0\n"
"       vmovupd (%%rcx,%%rax,1), %%ymm15\n"
"       vmovupd (%%r8,%%rax,1), %%ymm0\n"
"       vmovupd (%%r8,%%rax,1), %%ymm15\n"
"       vmovupd (%%rcx,%%r8,1), %%ymm0\n"
"       vmovupd (%%rcx,%%r8,1), %%ymm15\n"
"       vmovupd (%%r9,%%r8,1), %%ymm0\n"
"       vmovupd (%%r9,%%r8,1), %%ymm15\n"
"       vmovupd 42(%%rcx,%%rax,1), %%ymm0\n"
"       vmovupd 42(%%rcx,%%rax,1), %%ymm15\n"
"       vmovupd 42(%%r8,%%rax,1), %%ymm0\n"
"       vmovupd 42(%%r8,%%rax,1), %%ymm15\n"
"       vmovupd 42(%%rcx,%%r8,1), %%ymm0\n"
"       vmovupd 42(%%rcx,%%r8,1), %%ymm15\n"
"       vmovupd 42(%%r9,%%r8,1), %%ymm0\n"
"       vmovupd 42(%%r9,%%r8,1), %%ymm15\n"
"       vmovupd 128(%%rcx,%%rax,1), %%ymm0\n"
"       vmovupd 128(%%rcx,%%rax,1), %%ymm15\n"
#endif

#include "split_complex_mul_acc_simd_asm_m256d.hpp"

template <typename Triangle, typename TValue, typename TScale>
inline
__attribute__((flatten))
__attribute__((noinline))
void linear_complex_mul_acc(
        const TValue & value0,
        const TScale & scale0,
        Triangle & triangle,
        const size_t p)
{
    typedef typename Triangle::value_type complex_type;
    typedef typename complex_type::value_type simd_type;
    typedef typename Vec_traits<simd_type>::scalar_type scalar_type;
    Vec_traits<simd_type> VT;
    const size_t s = VT.VecSize;
    size_t n = (((p + 1) * (p + 2)) / 2) * 2 * s;  // scalar elements
    n -= n % (4 * s);
    n /= 4;  // split into 4 areas: reA, imA, reB, imB

    scalar_type * base = reinterpret_cast<scalar_type *>(&triangle(0, 0));

    scalar_type * pra = base;
    scalar_type * pia = pra + n;
    scalar_type * prb = pia + n;
    scalar_type * pib = prb + n;

    simd_type are = VT.zero();
    simd_type aim = VT.zero();

#if 0
    split_complex_mul_acc(are, aim, pra, pia, prb, pib, n);
#else
    split_complex_mul_acc_simd_asm(are, aim, pra, pia, prb, pib, n / s);
#endif

    triangle(0, 0) += complex_type(are, aim);
}

struct do_linear_complex_mul_acc
{
    template <typename Real4, typename Triangle>
    void operator () (
        const Real4 & xyzq,
        Triangle & triangle,
        const size_t p)
    {
        typedef typename Real4::value_type real_type;
        typedef typename Triangle::value_type complex_type;
        complexconst<complex_type> cc;
        real_type stuff = (xyzq.z + xyzq.q) * (xyzq.x - xyzq.y);
        complex_type cmplx = cc.one() * xyzq.x + cc.i() * xyzq.y;
        linear_complex_mul_acc(cmplx, stuff, triangle, p);
    }
};

#endif
// vim: et:ts=4:sw=4
