/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#ifndef _AnBe_FMM_MICROBENCH_linear_accumulate_hpp_
#define _AnBe_FMM_MICROBENCH_linear_accumulate_hpp_

#include "Vec.h"


template <typename T>
struct complexconst
{
    Vec_traits<T> VT;

    T zero()
    { return VT.same(0); }
    T one()
    { return VT.same(1); }
    T i()
    { return VT.same(-1); }
};

template <typename T>
struct complexconst<std::complex<T> >
{
    Vec_traits<T> VT;

    std::complex<T> zero()
    { return std::complex<T>(VT.same(0), VT.same(0)); }
    std::complex<T> one()
    { return std::complex<T>(VT.same(1), VT.same(0)); }
    std::complex<T> i()
    { return std::complex<T>(VT.same(0), VT.same(1)); }
};

/*
template <typename T>
T get_cmplx_i()
{
    Vec_traits<T> VT;
    return VT.same(-1);
}

template <typename T>
std::complex<T> get_cmplx_i<std::complex<T> >()
{
    Vec_traits<T> VT;
    return std::complex<T>(VT.same(0), VT.same(1));
}
*/

template <typename Triangle, typename TValue, typename TScale>
inline
__attribute__((flatten))
__attribute__((noinline))
void linear_accumulate(
        const TValue & value0,
        const TScale & scale0,
        Triangle & triangle,
        const size_t p)
{
    TScale scale = scale0 + scale0;
    TValue value = value0 + value0;

    typename Triangle::value_type *begin, *end;

    begin = &triangle(0, 0);
    end = begin + ((p + 1) * (p + 2) / 2);

    for (auto p = begin; p != end; ++p)
        *p += value * scale;
}

struct do_linear_accumulate
{
    template <typename Real4, typename Triangle>
    void operator () (
        const Real4 & xyzq,
        Triangle & triangle,
        const size_t p)
    {
        typedef typename Real4::value_type real_type;
        typedef typename Triangle::value_type complex_type;
        complexconst<complex_type> cc;
//        complex_type cmplx_i = get_cmplx_i<complex_type>();
//        complex_type cmplx_0 = cmplx_i - cmplx_i;
//        complex_type cmplx_1 = cmplx_0 - cmplx_i * cmplx_i;
        real_type stuff = (xyzq.z + xyzq.q) * (xyzq.x - xyzq.y);
        complex_type cmplx = cc.one() * xyzq.x + cc.i() * xyzq.y;
        linear_accumulate(cmplx, stuff, triangle, p);
    }
};

#endif
// vim: et:ts=4:sw=4
