/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#ifndef _AnBe_FMM_MICROBENCH_muladdmul_hpp_
#define _AnBe_FMM_MICROBENCH_muladdmul_hpp_

#include "Vec.h"


template <typename Triangle, typename TValue, typename TScale>
inline
__attribute__((flatten))
__attribute__((noinline))
void linear_muladdmul(
        const TValue & value0,
        const TScale & scale0,
        Triangle & triangle,
        const size_t p)
{
    TScale scale = scale0 + scale0;
    TValue value = value0 + value0;

    TScale xxx = scale * scale;
    TScale xx = scale - scale;
    TScale x = scale;

    TScale A = xxx - xx;
    TScale B = xx * xxx;
    TScale C = B - A;
    TScale D = -C * xx;

    size_t n = ((p + 1) * (p + 2)) / 2;

    for (size_t i = 0; i < n; ++i) {
        x = A * (B * xx - C * xxx);
        B += D;
        xxx = xx;
        xx = x;
    }
    triangle(p, p) += value * x;
}

struct do_linear_muladdmul
{
    template <typename Real4, typename Triangle>
    void operator () (
        const Real4 & xyzq,
        Triangle & triangle,
        const size_t p)
    {
        typedef typename Real4::value_type real_type;
        typedef typename Triangle::value_type complex_type;
        complexconst<complex_type> cc;
        real_type stuff = (xyzq.z + xyzq.q) * (xyzq.x - xyzq.y);
        complex_type cmplx = cc.one() * xyzq.x + cc.i() * xyzq.y;
        linear_muladdmul(cmplx, stuff, triangle, p);
    }
};

#endif
// vim: et:ts=4:sw=4
