/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#ifndef _AnBe_FMM_MICROBENCH_triangle_accumulate_hpp_
#define _AnBe_FMM_MICROBENCH_triangle_accumulate_hpp_

template <typename Triangle, typename TValue, typename TScale>
inline
__attribute__((flatten))
__attribute__((noinline))
void triangle_accumulate(
        const TValue & value0,
        const TScale & scale0,
        Triangle & triangle,
        const size_t p)
{
    TScale scale = scale0 + scale0;
    TValue value = value0 + value0;

    for (size_t m = 0; m < p; ++m) {
        triangle(m, m) += value * scale;
        triangle(m + 1, m) += value * scale;
        for (size_t l = m + 2; l <= p; ++l) {
            triangle(l, m) += value * scale;
        }
    }
    triangle(p, p) += value * scale;
}

struct do_triangle_accumulate
{
    template <typename Real4, typename Triangle>
    void operator () (
        const Real4 & xyzq,
        Triangle & triangle,
        const size_t p)
    {
        typedef typename Real4::value_type real_type;
        typedef typename Triangle::value_type complex_type;
        real_type stuff = (xyzq.z + xyzq.q) * (xyzq.x - xyzq.y);
        triangle_accumulate(complex_type(xyzq.x, xyzq.y), stuff, triangle, p);
    }
};

#endif
// vim: et:ts=4:sw=4
