#ifndef _DaHa_DependencyManager_
#define _DaHa_DependencyManager_

#include "DependencyCounter.hpp"
#include "EventDispatcher/sdfd.hpp"


template <typename sdfd, typename fmm_handle_type>
class DependencyManager
{
    using DataFlowDispatcher = sdfd;
    using BoxID = typename fmm_handle_type::boxid_type;

public:
    DependencyManager(fmm_handle_type &fmm_handle) : fmm_handle_(fmm_handle), dc_(fmm_handle)
    {
    }


    template <typename TaskFactory>
    void ResolveFinalOmega(BoxID &bid, TaskFactory &tf)
    {
        if (bid.d() > 0)
        {
            BoxID parent_bid = bid.parent();
            dc_.template dispatch<OMEGA_Target>(parent_bid.sfcindex(), parent_bid.d(), tf);
            dc_.template dispatch<OMEGA_Source>(bid.sfcindex(), bid.d(), tf);
        }
        if (bid.d() == 0 && !fmm_handle_.simulationbox().open_boundary_conditions())
        {
            tf.CreateAndEnqueueTask(tf.lattice, bid.sfcindex(), bid.d());
        }
    }

    template <typename TaskFactory, typename ObjectWrapperTreeType>
    void ResolvePartialMu(BoxID &bid, TaskFactory &tf, ObjectWrapperTreeType &)
    {
        if (bid.d() == fmm_handle_.depth())
        {
            dc_.template dispatch<MU_Lowest>(bid.sfcindex(), bid.d(), tf, fmm_handle_);
        }
        else
        {
            dc_.template dispatch<MU_Source>(bid.sfcindex(), bid.d(), tf, fmm_handle_);
        }
    }


    DependencyCounters<DataFlowDispatcher, fmm_handle_type> &dc()
    {
        return dc_;
    }

    void ResetDepsCounter()
    {
        dc_.Reset();
    }

private:
    fmm_handle_type &fmm_handle_;
    DependencyCounters<DataFlowDispatcher, fmm_handle_type> dc_;
};


#endif
