#ifndef _DaHa_CallbackImpl_
#define _DaHa_CallbackImpl_

#include <cstddef>

#include "src/callback.hpp"

#include "EventDispatcherEnumerations.hpp"


template <HierarchyLevels hierarchy_level, Events triggered_event>
struct CallbackImpl<P2M_Task, hierarchy_level, triggered_event>
{
    template <typename TaskFactory>
    static void Call(size_t /*unused*/, size_t /*unused*/, TaskFactory & /*unused*/)
    {
    }
};

template <HierarchyLevels hierarchy_level, Events triggered_event>
struct CallbackImpl<M2M_Task, hierarchy_level, triggered_event>
{
    template <typename TaskFactory>
    static void Call(size_t id, size_t d, TaskFactory &task_factory)
    {
        task_factory.CreateAndEnqueueTask(task_factory.m2m, id, d);
    }
};


template <HierarchyLevels hierarchy_level, Events triggered_event>
struct CallbackImpl<M2L_Task, hierarchy_level, triggered_event>
{
    template <typename TaskFactory>
    static void Call(size_t id, size_t d, TaskFactory &task_factory)
    {
        // FIXME: we have to filter unnedded M2L Calls in DependencyManager!
        if (d > 0)
        {
            task_factory.CreateAndEnqueueTask(task_factory.m2l, id, d);
        }
    }
};


template <HierarchyLevels hierarchy_level, Events triggered_event>
struct CallbackImpl<L2L_Task, hierarchy_level, triggered_event>
{
    template <typename TaskFactory, typename FMMHandle>
    static void Call(size_t id, size_t d, TaskFactory &tf, FMMHandle &fmm_handle)
    {
        if (fmm_handle.simulationbox().open_boundary_conditions())
        {
            if (d > 1 &&                 // shift downwards starting from d=2
                d < fmm_handle.depth())  // no downwards on the last level
            {
                tf.CreateAndEnqueueTask(tf.l2l, id, d);
            }
        }
        else
        {
            if (d < fmm_handle.depth())
            {
                tf.CreateAndEnqueueTask(tf.l2l, id, d);
            }
        }
    }
};


template <HierarchyLevels hierarchy_level, Events triggered_event>
struct CallbackImpl<L2P_Task, hierarchy_level, triggered_event>
{
    template <typename TaskFactory, typename FMMHandle>
    static void Call(size_t id, size_t d, TaskFactory &tf, FMMHandle & /*unused*/)
    {
        tf.CreateAndEnqueueTask(tf.l2p, id, d);
    }
};
#endif
