#ifndef _sdfd_dependency_counter_
#define _sdfd_dependency_counter_

#include <atomic>
#include<tuple>

#include "../EventDispatcherEnumerations.hpp"

template <size_t default_value, Events event>
class DependencyCounter
{
public:
    void Reset()
    {
        counter = default_value;
    }

    // needs to be public member variable, otherwise thread safety is hard!
    bool DecrAndCheck()
    {
        return --counter == 0;
    }

    std::atomic<size_t> counter{default_value};
    const static Events EVENT = event;
};

template <Events event>
class DependencyCounter<1, event>
{
public:
    void Reset()
    {
    }
    bool DecrAndCheck()
    {
        return true;
    }
    const static Events EVENT = event;
};
template <typename T, size_t PADDING = 64>
class counter_tuple_wrapper : public T
{
public:
    using size_type = size_t;
    using wrapped_type = T;

    static constexpr size_t additional_padding_size_ = PADDING - sizeof(T) % PADDING;
    uint8_t padding_[additional_padding_size_];


    template <size_t current = 0>
    typename std::enable_if<current == std::tuple_size<T>::value, void>::type Reset()
    {
    }

    template <size_t current = 0>
    typename std::enable_if<current != std::tuple_size<T>::value, void>::type Reset()
    {
        std::get<current>(get_tuple()).Reset();
        Reset<current + 1>();
    }

private:
    T &get_tuple()
    {
        return static_cast<T &>(*this);
    }
};
#endif
