#ifndef _sdfd_event_listener_
#define _sdfd_event_listener_

#include <initializer_list>
#include <utility>

#include "listener.hpp"

#include "unused_expression_helper.hpp"

/**
 * The EventListener consists of an Event listening on and the registered Listeners.
 * Whenever an Event is dispatched it will be filtered (with SFINAE) to be equal to the listened
 * Event and the dispatch of all registered Listeners will be called.
 */
template <size_t Counter, Events Event, typename... Listeners>
class EventListener
{
public:
    // Exposes the listened Event to other classes and objects (required for type construction)
    const static Events EVENT = Event;
    const static size_t COUNTER = Counter;

    template <HierarchyLevels triggered_hierarchy_level, Events triggered_event, typename... Args>
    static typename std::enable_if<triggered_event == Event, void>::type dispatch(Args &&... args)
    {
        /**
         * The initalizer list is used to get the right context for expanding the tempalte pack and
         * call the static function.
         * For details check:
         * https://stackoverflow.com/questions/31204084/is-it-possible-to-call-static-method-form-variadic-template-type-parameter
         */
        unused_expression(std::initializer_list<int>{
                (Listeners::template dispatch<triggered_hierarchy_level, triggered_event>(
                         std::forward<Args>(args)...),
                 0  // Necessary because return of Resolve is void and we need at least one int here
                 )...});
    }

    template <HierarchyLevels triggered_hierarchy_level, Events triggered_event, typename... Args>
    static typename std::enable_if<triggered_event != Event, void>::type
    dispatch(Args &&... /*unused*/)
    {
        // This is an NOOP because the triggered event is not equal to the listening event
    }
};


#endif
