#ifndef _sdfd_event_listener_container_
#define _sdfd_event_listener_container_

#include <initializer_list>
#include <tuple>
#include <utility>

#include "dependency_counter.hpp"
#include "event_listener.hpp"
#include "listener.hpp"

#include "unused_expression_helper.hpp"

/**
 * The EventListenerContainer consists out of pairs of events and related listeners (EventListener).
 * Those EventListeners contain an Event and a template pack of Listeners.
 * The EventListenerContainer dispatches Events to the EventListeners without filtering on Events.
 * The EventListener is responsible for filtering out other events which are not of interest for
 * this EventListener.
 */

template <typename... EventListeners>
class EventListenerContainer
{
public:
    // This generates a corresponding tuple type with all counters required for the registered
    // Events.
    using dependency_counter_tuple_type =
            std::tuple<DependencyCounter<EventListeners::COUNTER, EventListeners::EVENT>...>;

    /**
     * Dispatches an Event to the EventListeners registered.
     *
     * The Event has to be set as template parameter as well as the quantity for which the event is
     * triggered.
     * Function parameters are used to forward parameters to the Callbacks of the Listener.
     */
    template <HierarchyLevels triggered_hierarchy_level, Events triggered_event, typename... Args>
    static void dispatch(Args &&... args)
    {
        /**
         * The initalizer list is used to get the right context for expanding the tempalte pack and
         * call the static function.
         * For details check:
         * https://stackoverflow.com/questions/31204084/is-it-possible-to-call-static-method-form-variadic-template-type-parameter
         */
        unused_expression(std::initializer_list<int>{
                (EventListeners::
                         template dispatch<triggered_hierarchy_level, triggered_event, Args...>(
                                 std::forward<Args>(args)...),
                 0  // Necessary because return of Resolve is void and we need at least one int here
                 )...});
    }
};


#endif
