#ifndef _DaHa_Executor_
#define _DaHa_Executor_

#include "MultiQueue.hpp"
#include "ThreadDormitory.hpp"
#include "parallelization/ThreadIdentifier.hpp"
#include "fmsolvr/container/simple_vector.hpp"

#include <atomic>
#include <memory>
template <typename MultiQueueType>
class Executor
{
public:
    Executor() = delete;
    Executor(const Executor &) = delete;
    Executor(Executor &&) = delete;
    Executor &operator=(const Executor &) = delete;
    Executor &operator=(Executor &&) = delete;

    virtual ~Executor() = default;

    Executor(
            unsigned int main_que_index,
            fmsolvr::simple_vector<MultiQueueType> &queues,
            ThreadDormitory &thread_dormitory)
            : main_que_index_(main_que_index), queues_(queues), thread_dormitory_(thread_dormitory)
    {
    }

    void InitializeThreadLocal()
    {
        _THIS_THREAD = ThreadIdentifier{main_que_index_};
    }

    void Run()
    {
        InitializeThreadLocal();

        while (!join_)
        {
            RunOne();
            thread_dormitory_.trySleep();
        }
    }

    void RunOne()
    {
        if (!queues_[main_que_index_].empty())
        {
            queues_[main_que_index_].execute_front();
        }
        else if (steal_)
        {
            // steal work from que of other thread
            unsigned int i = (main_que_index_ + 1) % queues_.size();
            while (!queues_[i].stealable() && i != main_que_index_)
            {
                i = (i + 1) % queues_.size();
            }
            queues_[i].execute_front();
        }
    }

    void RunAll()
    {
        while (!queues_[main_que_index_].empty())
        {
            RunOne();
        }
    }

    void Finalize()
    {
        join_ = true;
    }

    bool empty()
    {
        return queues_[main_que_index_].empty();
    }

    void EnableWorkStealing()
    {
        steal_ = true;
    }

    void DisableWorkStealing()
    {
        steal_ = false;
    }

private:
    std::atomic<bool> join_ = {false};
    std::atomic<bool> steal_ = {false};

    unsigned int main_que_index_;
    fmsolvr::simple_vector<MultiQueueType> &queues_;
    ThreadDormitory &thread_dormitory_;
};

#endif
