#ifndef _DaHa_LoadBalancer_
#define _DaHa_LoadBalancer_

#include <cmath>
#include <list>
#include <memory>

#include "LoadBalancerHelper.hpp"
#include "MultiQueue.hpp"
#include "parallelization/Task.hpp"
#include "fmsolvr/container/simple_vector.hpp"


template <typename FMMHandle, typename MultiQueueType>
class LoadBalancer
{
public:
    using size_type = typename fmsolvr::simple_vector<MultiQueueType>::size_type;

    LoadBalancer(FMMHandle &fmm_handle, fmsolvr::simple_vector<MultiQueueType> &queues)
            : fmm_handle_(fmm_handle), queues_(queues)
    {
        auto depth = fmm_handle.depth();
        for (unsigned d = 0; d <= depth; ++d)
        {
            equal_box_dists.push_back(load_balancer::details::BoxEqualDist(d, NumQueues()));
        }
    }

    ~LoadBalancer()
    {
    }

    template <typename TaskType>
    void SelectQueue(TaskType *task) const
    {
        queues_[GetQueueId(task)].push_back(task);
    }

    template <typename... ProcessorArgs, typename TreePartitionType>
    void SelectQueue(Task<M2LProcessor<ProcessorArgs...>, TreePartitionType> *task) const
    {
        queues_[GetQueueId(task)].push_front(task);
    }

    size_type NumQueues() const
    {
        return queues_.size();
    }

private:
    FMMHandle &fmm_handle_;

    fmsolvr::simple_vector<MultiQueueType> &queues_;

    std::vector<load_balancer::details::EqualDist> equal_box_dists;

    template <
            template <typename...> class Processor,
            typename... ProcessorArgs,
            typename TreePartitionType>
    int GetQueueId(Task<Processor<ProcessorArgs...>, TreePartitionType> *) const
    {
        Processor<ProcessorArgs...>::THIS_FUNCTION_IS_NOT_IMPLEMENTED;
        return 0;
    }

    template <typename... ProcessorArgs, typename TreePartitionType>
    int GetQueueId(Task<P2MProcessor<ProcessorArgs...>, TreePartitionType> *task) const
    {
        return equal_box_dists[task->GetDepth()].GetBlockId(task->GetBoxId());
    }

    template <typename... ProcessorArgs, typename TreePartitionType>
    int GetQueueId(Task<P2PProcessor<ProcessorArgs...>, TreePartitionType> *task) const
    {
        return equal_box_dists[task->GetDepth()].GetBlockId(task->GetBoxId());
    }

    template <typename... ProcessorArgs, typename TreePartitionType>
    int GetQueueId(Task<M2LProcessor<ProcessorArgs...>, TreePartitionType> *task) const
    {
        return equal_box_dists[task->GetDepth()].GetBlockId(task->GetBoxId());
    }

    template <typename... ProcessorArgs, typename TreePartitionType>
    int GetQueueId(Task<L2LProcessor<ProcessorArgs...>, TreePartitionType> *task) const
    {
        return equal_box_dists[task->GetDepth()].GetBlockId(task->GetBoxId());
    }

    template <typename... ProcessorArgs, typename TreePartitionType>
    int GetQueueId(Task<L2PProcessor<ProcessorArgs...>, TreePartitionType> *task) const
    {
        return equal_box_dists[task->GetDepth()].GetBlockId(task->GetBoxId());
    }
    template <typename... ProcessorArgs, typename TreePartitionType>
    int GetQueueId(Task<M2MProcessor<ProcessorArgs...>, TreePartitionType> *task) const
    {
        return equal_box_dists[task->GetDepth()].GetBlockId(task->GetBoxId());
    }
    template <typename... ProcessorArgs, typename TreePartitionType>
    int GetQueueId(Task<LatticeProcessor<ProcessorArgs...>, TreePartitionType> *) const
    {
        return 0;
    }
};


#endif
