#ifndef _DaHa_LoadBalancerHelper_
#define _DaHa_LoadBalancerHelper_

#include <cmath>
#include <cstddef>
#include <iostream>
#include <vector>


namespace load_balancer
{
namespace details
{
class EqualDist
{
public:
    EqualDist(size_t n, size_t num_blocks)
            : n(n),
              num_blocks(num_blocks),
              bs_begin(static_cast<size_t>(ceil(1. * n / num_blocks))),
              bs_end(bs_begin - 1)
    {
        // the threshold is the first block (id) with the lower blocksize (bs_end). In case that
        // n%num_blocks == 0 we only have bs_begin
        bs_threshold = (n % num_blocks) == 0 ? num_blocks : n % num_blocks;
    }

    int GetBlockId(size_t id) const
    {
        if (id < bs_begin * bs_threshold)
        {
            return id / bs_begin;
        }
        {
            return (id - bs_begin * bs_threshold) / bs_end + bs_threshold;
        }
    }

    std::vector<size_t> GetBSVector() const
    {
        std::vector<size_t> bs_vec(num_blocks);
        for (unsigned int i = 0; i < num_blocks; ++i)
        {
            if (i < bs_threshold)
            {
                bs_vec[i] = bs_begin;
            }
            else
            {
                bs_vec[i] = bs_end;
            }
        }
        return bs_vec;
    }


    friend std::ostream &operator<<(std::ostream &os, const EqualDist &ed);

private:
    size_t n, num_blocks, bs_begin, bs_end, bs_threshold;
};

inline
std::ostream &operator<<(std::ostream &os, const EqualDist &ed)
{
    os << "n: " << ed.n << " blocks: " << ed.num_blocks << " begin size: " << ed.bs_begin
       << " end bs:  " << ed.bs_end << " threshold: " << ed.bs_threshold;
    return os;
}

inline
EqualDist BoxEqualDist(size_t d, size_t num_blocks)
{
    return EqualDist{static_cast<size_t>(std::pow(8, d)), num_blocks};
}


}  // namespace details

}  // namespace load_balancer

#endif
