#ifndef _DaHa_MCSLock_
#define _DaHa_MCSLock_

#include <atomic>
#include <iostream>

struct alignas(64) MCSLocalLock
{
    std::atomic<MCSLocalLock *> next_{nullptr};
    std::atomic_flag locked_;
};


class alignas(64) MCSLock
{
    std::atomic<MCSLocalLock *> tail_{nullptr};

    // We use a getter method to lazy initialize the static variable on the first call.
    // https://stackoverflow.com/questions/18860895/how-to-initialize-static-members-in-the-header
    MCSLocalLock &GetLocalLock()
    {
        static thread_local MCSLocalLock local_lock_;
        return local_lock_;
    }

public:
    void lock()
    {
        GetLocalLock().next_ = nullptr;
        MCSLocalLock *predecessor = tail_.exchange(
                &GetLocalLock());    // exchange the tail with the local lock of the thread
        if (predecessor != nullptr)  // lock is currently locked, tail was not null
        {
            GetLocalLock().locked_.test_and_set();  // set the locked state of local lock to true
            predecessor->next_ = &GetLocalLock();   // link to the predecessor
            while (GetLocalLock().locked_.test_and_set())
            {
                ;  // spin on local variable
            }
        }
    }

    void unlock()
    {
        if (GetLocalLock().next_ == nullptr)  // no known successor
        {
            // for the compare and exchange we need an L-value reference to the local lock pointer!
            // Therefore we can not use the pointer on the fly as R-value reference and need a temp
            // variable with the pointer.
            MCSLocalLock *local_lock_ptr = &GetLocalLock();
            // Attention: local_lock_ptr is exchanged after the compare_exchange_strong as well, so
            // we can't use it anymore!
            // if tail pointed to local lock, no other thread wants to acquire the lock
            if (tail_.compare_exchange_strong(local_lock_ptr, nullptr))
            {
                return;
            }

            // if other thread wants to acquire the lock, we have to wait until the next pointer is
            // set by the other thread
            while (GetLocalLock().next_ == nullptr)
            {
                ;  // spin on local variable
            }
        }
        GetLocalLock().next_.load()->locked_.clear();  // unlock next
    }
};
#endif  // _DaHa_MCSLock_
