#ifndef _LaMo_MultiQueue_
#define _LaMo_MultiQueue_

#include "MultiQueueDetails.hpp"
#include "Task.hpp"

using namespace multi_queue::details;

template <
        template <template <typename...> class> class ProcessorTypeConstructor,
        typename TreePartitionType>
class MultiQueue
{
public:
    template <template <typename...> class ProcessorType>
    using TaskTypeConstructor = Task<ProcessorTypeConstructor<ProcessorType>, TreePartitionType>;

    template <typename task_type>
    void push_back(task_type *task)
    {
        auto &queue_with_mutex = tuple_wrapper.GetQueue(task);

        queue_with_mutex.mutex.lock();
        queue_with_mutex.queue.push_back(task);
        queue_with_mutex.mutex.unlock();
    }

    template <typename task_type>
    void push_front(task_type *task)
    {
        auto &queue_with_mutex = tuple_wrapper.GetQueue(task);

        queue_with_mutex.mutex.lock();
        queue_with_mutex.queue.push_front(task);
        queue_with_mutex.mutex.unlock();
    }


    // remove task from que and execute it
    void execute_front()
    {
        ExecuteNextOrdered();
    }

    bool empty() const
    {
        return tuple_wrapper.empty();
    }
    bool stealable() const
    {
        return !tuple_wrapper.empty();
    }


private:
    using tuple_wrapper_type = queue_tuple_wrapper<
            TaskTypeConstructor,
            P2MProcessor,
            M2MProcessor,
            M2LProcessor,
            L2LProcessor,
            L2PProcessor,
            P2PProcessor>;


    tuple_wrapper_type tuple_wrapper;

    template <typename QueueWithMutexType>
    void ExecuteQue(QueueWithMutexType &que)
    {
        typename QueueWithMutexType::TaskType *task = nullptr;
        que.mutex.lock();
        if (!que.queue.empty())
        {
            task = que.queue.front();
            que.queue.pop_front();
        }
        que.mutex.unlock();
        if (task != nullptr) {
            ExecuteTask(task);
            delete task;
        }
    }

    template <typename TaskType>
    void ExecuteTask(TaskType *task)
    {
        task->TaskType::execute();
    }

    void ExecuteTask(BaseTask *task)
    {
        task->execute();
    }


    template <int act_pos = 0>
    typename std::enable_if<act_pos != tuple_wrapper_type::tuple_size, void>::type
    ExecuteNextOrdered()
    {
        if (!tuple_wrapper.template GetQueue<act_pos>().queue.empty())
            ExecuteQue(tuple_wrapper.template GetQueue<act_pos>());
        else
            ExecuteNextOrdered<act_pos + 1>();
    }
    template <int act_pos = 0>
    typename std::enable_if<act_pos == tuple_wrapper_type::tuple_size, void>::type
    ExecuteNextOrdered()
    {
        return;
    }
};

#endif
