#ifndef _DaHa_MultiQueueDetails_
#define _DaHa_MultiQueueDetails_

#include <deque>
#include <functional>
#include <mutex>
#include <tuple>
#include <utility>
#include "Locks/MCSLock.hpp"
class BaseTask;

namespace multi_queue
{
namespace details
{
template <typename element_type>
class QueueWithMutex
{
public:
    using TaskType = element_type;

private:
    using queue_type = std::deque<TaskType *>;

public:
    queue_type queue;
    MCSLock mutex;

    bool empty() const
    {
        return queue.empty();
    }
};

template <
        template <template <typename...> class> class task_type_constructor,
        template <typename...> class... processor_template>
class queue_tuple_wrapper
{
private:
    using tuple_type = std::tuple<
            QueueWithMutex<task_type_constructor<processor_template>>...,
            QueueWithMutex<BaseTask>>;
    tuple_type queue_tuple_;
    template <typename que_type, int act = 0>
    static constexpr
            typename std::enable_if<act != std::tuple_size<tuple_type>::value - 1, int>::type
            TuplePos()
    {
        return std::is_same<que_type, typename std::tuple_element<act, tuple_type>::type>::value
                       ? act
                       : TuplePos<que_type, act + 1>();
    }
    template <typename que_type, int act = 0>
    static constexpr
            typename std::enable_if<act == std::tuple_size<tuple_type>::value - 1, int>::type
            TuplePos()
    {
        return act;
    }

public:
    const static int tuple_size = std::tuple_size<tuple_type>::value;

    template <typename task_type>
    typename std::tuple_element<TuplePos<QueueWithMutex<task_type>>(), tuple_type>::type &
    GetQueue(task_type * /*unused*/)
    {
        return std::get<queue_tuple_wrapper::TuplePos<QueueWithMutex<task_type>>()>(queue_tuple_);
    }

    template <std::size_t ID>
    typename std::tuple_element<ID, tuple_type>::type &GetQueue()
    {
        return std::get<ID>(queue_tuple_);
    }
    template <typename task_type>
    const QueueWithMutex<task_type> &GetQueue(task_type * /*unused*/) const
    {
        return std::get<queue_tuple_wrapper::TuplePos<QueueWithMutex<task_type>>()>(queue_tuple_);
    }

    template <std::size_t ID>
    const typename std::tuple_element<ID, tuple_type>::type &GetQueue() const
    {
        return std::get<ID>(queue_tuple_);
    }

    template <std::size_t act_pos = 0>
    typename std::enable_if<act_pos != queue_tuple_wrapper::tuple_size - 1, bool>::type
    empty() const
    {
        return GetQueue<act_pos>().empty() && empty<act_pos + 1>();
    }
    template <std::size_t act_pos = 0>
    typename std::enable_if<act_pos == queue_tuple_wrapper::tuple_size - 1, bool>::type
    empty() const
    {
        return GetQueue<act_pos>().empty();
    }

private:
};

}  // namespace details
}  // namespace multi_queue

#endif
