#ifndef _DaHa_ObjectWrapper_
#define _DaHa_ObjectWrapper_

#include <cstddef>

#include "parallelization/ObjectAccessProvider.hpp"
#include "parallelization/ThreadIdentifier.hpp"

/**
 * ObjectWrapper encapsulates all data required for parallelization related to wrapped Object.
 *
 * This comprises:
 *  - Dependency Counter
 *  - Access Manager (Locks)
 *
 * This should solve cache issues because data belonging to certain objects are encapsulated with
 * this wrapper
 */
template <typename ObjectType, typename ObjectAccessStrategy>
class ObjectWrapper
{
public:
    using value_type = ObjectType;
    using size_type = size_t;
    using object_access_provider_type = ObjectAccessProvider<value_type, ObjectAccessStrategy>;
    using object_access_type = typename object_access_provider_type::object_access_type;

    explicit ObjectWrapper(value_type &o) : obj_(o){};

    const object_access_type AcquireAccess()
    {
        return oap_.GetObjectAccess(obj_);
    }

private:
    value_type &obj_;
    object_access_provider_type oap_;
};

#endif
