#ifndef _DaHa_Task_
#define _DaHa_Task_

#include <list>
#include <memory>

#include "Processor.hpp"

class BaseTask
{
public:
    virtual void execute() = 0;
    virtual ~BaseTask() = default;
};


template <typename ProcessorType, typename TreePartitionType>
class Task : public BaseTask
{
public:
    Task(ProcessorType &exec, TreePartitionType tree_partition) : executor_(exec), tree_partition_(tree_partition){}

    /**
      \brief Executes the stored Executor of this Task.
    */
    void execute() override
    {
        executor_.execute(tree_partition_);
    }
    void debug()
    {
        std::cout << "first box " << tree_partition_.first() << " typr " << typeid(ProcessorType).name()
                  << std::endl;
    }

    size_t GetBoxId() const
    {
        // FIXME: do not return only first box of an interval
        return tree_partition_.first().sfcindex();
    }
    size_t GetDepth() const
    {
        return tree_partition_.first().d();
    }

private:
    /**
     * This needs to be a pointer because all abstraction is hidden in the inheritance of
     * Executors.
     * This avoids template parameters in this class (Task).
     */
    ProcessorType &executor_;
    TreePartitionType tree_partition_;

    friend std::ostream &operator<<(std::ostream &os, const Task<ProcessorType, TreePartitionType> &t)
    {
        os << "Task: (sfc_id: " << t.id << ", depth: " << t.d
           << ") ProcessorType: " << typeid(ProcessorType).name();
        return os;
    }
};

#endif
