#ifndef _DaHa_TaskFactory_
#define _DaHa_TaskFactory_

#include "LoadBalancer.hpp"
#include "Processor.hpp"
#include <map>

template <typename fmm_handle_type, typename mutex_container_type, typename dependency_manager>
class TaskFactory
{
public:
    using FMMHandle = fmm_handle_type;
    using DependencyManagerType = dependency_manager;
    using MutexContainerType = mutex_container_type;

    using TreePartitionType = typename FMMHandle::tree_partition_type;

    template <template <typename...> class ProcessorType>
    using ProcessorTypeConstructor =
            ProcessorType<FMMHandle, MutexContainerType, DependencyManagerType>;

    using MultiQueueType = MultiQueue<ProcessorTypeConstructor, TreePartitionType>;
    using LoadBalancerType = LoadBalancer<FMMHandle, MultiQueueType>;
    TaskFactory(
            FMMHandle &fmm_handle,
            MutexContainerType &mutex_container,
            LoadBalancerType &load_balancer,
            DependencyManagerType &dm)
            : p2m(fmm_handle, mutex_container, *this, dm),
              m2m(fmm_handle, mutex_container, *this, dm),
              m2l(fmm_handle, mutex_container, *this, dm),
              l2l(fmm_handle, mutex_container, *this, dm),
              l2p(fmm_handle, mutex_container, *this, dm),
              p2p(fmm_handle, mutex_container, *this, dm),
              lattice(fmm_handle, mutex_container, *this, dm),
              load_balancer_(load_balancer){}

    ProcessorTypeConstructor<P2MProcessor> p2m;
    ProcessorTypeConstructor<M2MProcessor> m2m;
    ProcessorTypeConstructor<M2LProcessor> m2l;
    ProcessorTypeConstructor<L2LProcessor> l2l;
    ProcessorTypeConstructor<L2PProcessor> l2p;
    ProcessorTypeConstructor<P2PProcessor> p2p;
    ProcessorTypeConstructor<LatticeProcessor> lattice;

    template <typename ProcessorType>
    void CreateAndEnqueueTask(ProcessorType &processor, TreePartitionType tree_partition)
    {
        load_balancer_.SelectQueue(MakeTask(processor, tree_partition));
    }

    template <typename ProcessorType>
    void CreateAndEnqueueTask(ProcessorType &processor, size_t id, size_t d)
    {
        load_balancer_.SelectQueue(MakeTask(processor, TreePartitionType(id, d)));
    }
private:
    LoadBalancerType &load_balancer_;

    template <typename ProcessorType>
    Task<ProcessorType, TreePartitionType> *MakeTask(ProcessorType &processor, TreePartitionType tree_partition)
    {
        return new Task<ProcessorType, TreePartitionType>(processor, tree_partition);
    }
};

#endif
