#ifndef _DaHa_ThreadingWrapper_
#define _DaHa_ThreadingWrapper_

#include <thread>

template <typename ExecutorType>
class ThreadingWrapper
{
public:
    using id_type = std::thread::id;

    ThreadingWrapper() = delete;
    ThreadingWrapper(const ThreadingWrapper &) = delete;
    ThreadingWrapper(ThreadingWrapper &&) = delete;
    ThreadingWrapper &operator=(const ThreadingWrapper &) = delete;
    ThreadingWrapper &operator=(ThreadingWrapper &&) = delete;

    explicit ThreadingWrapper(ExecutorType &executor, bool primary = false) : executor_(executor)
    {
        if (primary)
        {
            return;
        }
        thread_ = std::thread([&executor] { executor.Run(); });
    }


    ~ThreadingWrapper()
    {
        Join();
    }

    std::thread::id GetId() const
    {
        return thread_.get_id();
    }

    static std::thread::id GetThisId()
    {
        return std::this_thread::get_id();
    }

private:
    ExecutorType &executor_;
    std::thread thread_{};

    void Join()
    {
        executor_.Finalize();

        if (thread_.joinable())
        {
            thread_.join();
        }
    }
};

#endif
