/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#include "fmsolvr/default_alloc.hpp"
#include "fmsolvr/operator/rotation/rotation_types.hpp"
#include "fmsolvr/util/float128.hpp"

#include "bulk_ops_impl.hpp"

namespace fmsolvr {

namespace bulk {

namespace inst_float {

typedef float Real;
typedef typename fmsolvr::default_allocator<Real>::type Alloc;
typedef typename fmsolvr::RotationOMuga<Real, Alloc>::CoeffMatrix CoeffMatrix;
typedef fmsolvr::RotationDataTypes<Real, Alloc> RotDataTypes;
typedef typename RotDataTypes::Rot_Operator RotationOperator;
typedef typename RotDataTypes::M2M_Operator M2Mp3_Operator;
typedef typename RotDataTypes::M2L_Operator M2Lp3_Operator;
typedef typename RotDataTypes::L2L_Operator L2Lp3_Operator;
typedef typename RotDataTypes::template ScratchTypeGenerator<typename CoeffMatrix::value_type>::Scratch Scratch;

typedef bulk::BulkRotOp<CoeffMatrix, RotationOperator, M2Lp3_Operator, CoeffMatrix> bulkdataM2L;
typedef const bulkdataM2L * M2LBulkIterator;

}

namespace inst_double {

typedef double Real;
typedef typename fmsolvr::default_allocator<Real>::type Alloc;
typedef typename fmsolvr::RotationOMuga<Real, Alloc>::CoeffMatrix CoeffMatrix;
typedef fmsolvr::RotationDataTypes<Real, Alloc> RotDataTypes;
typedef typename RotDataTypes::Rot_Operator RotationOperator;
typedef typename RotDataTypes::M2M_Operator M2Mp3_Operator;
typedef typename RotDataTypes::M2L_Operator M2Lp3_Operator;
typedef typename RotDataTypes::L2L_Operator L2Lp3_Operator;
typedef typename RotDataTypes::template ScratchTypeGenerator<typename CoeffMatrix::value_type>::Scratch Scratch;

typedef bulk::BulkRotOp<CoeffMatrix, RotationOperator, M2Lp3_Operator, CoeffMatrix> bulkdataM2L;
typedef const bulkdataM2L * M2LBulkIterator;

}

namespace inst_longdouble
{

typedef long double Real;
typedef typename fmsolvr::default_allocator<Real>::type Alloc;
typedef typename fmsolvr::RotationOMuga<Real, Alloc>::CoeffMatrix CoeffMatrix;
typedef fmsolvr::RotationDataTypes<Real, Alloc> RotDataTypes;
typedef typename RotDataTypes::Rot_Operator RotationOperator;
typedef typename RotDataTypes::M2M_Operator M2Mp3_Operator;
typedef typename RotDataTypes::M2L_Operator M2Lp3_Operator;
typedef typename RotDataTypes::L2L_Operator L2Lp3_Operator;
typedef typename RotDataTypes::template ScratchTypeGenerator<typename CoeffMatrix::value_type>::Scratch Scratch;

typedef bulk::BulkRotOp<CoeffMatrix, RotationOperator, M2Lp3_Operator, CoeffMatrix> bulkdataM2L;
typedef const bulkdataM2L * M2LBulkIterator;

}

// explicit instantiation
template void M2Lp3(inst_float::M2LBulkIterator first, inst_float::M2LBulkIterator last, size_t p, inst_float::Scratch &);
template void M2Lp3(inst_double::M2LBulkIterator first, inst_double::M2LBulkIterator last, size_t p, inst_double::Scratch &);
template void M2Lp3(inst_longdouble::M2LBulkIterator first, inst_longdouble::M2LBulkIterator last, size_t p, inst_longdouble::Scratch &);


#ifdef HAVE_FLOAT128

namespace inst_float128
{

typedef __float128 Real;
typedef typename fmsolvr::default_allocator<Real>::type Alloc;
typedef typename fmsolvr::RotationOMuga<Real, Alloc>::CoeffMatrix CoeffMatrix;
typedef fmsolvr::RotationDataTypes<Real, Alloc> RotDataTypes;
typedef typename RotDataTypes::Rot_Operator RotationOperator;
typedef typename RotDataTypes::M2M_Operator M2Mp3_Operator;
typedef typename RotDataTypes::M2L_Operator M2Lp3_Operator;
typedef typename RotDataTypes::L2L_Operator L2Lp3_Operator;
typedef typename RotDataTypes::template ScratchTypeGenerator<typename CoeffMatrix::value_type>::Scratch Scratch;

typedef bulk::BulkRotOp<CoeffMatrix, RotationOperator, M2Lp3_Operator, CoeffMatrix> bulkdataM2L;
typedef const bulkdataM2L * M2LBulkIterator;

}

template void M2Lp3(inst_float128::M2LBulkIterator first, inst_float128::M2LBulkIterator last, size_t p, inst_float128::Scratch &);

#endif  // HAVE_FLOAT128

}  // namespace bulk

}  // namespace fmsolvr

// vim: et:ts=4:sw=4
