#
# This file is part of FMSOLVR, a Fast Multipole Solver for MD.
#
# Copyright © 2019 Andreas Beckmann
#
# FMSOLVR is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 2.1
# of the License, or (at your option) any later version.
#
# FMSOLVR is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with FMSOLVR.  If not, see
# <https://www.gnu.org/licenses/>.
#
ALGOS		?= 0 1 3 4 5 2
THREADS		?= 1 2 3 4 5 6 7 8 9
THREADS_ALGO_0	?= 0
THREADS_ALGO_1	?= $(THREADS)
THREADS_ALGO_2	?= 0
THREADS_ALGO_3	?= $(THREADS)
THREADS_ALGO_4	?= $(THREADS)
THREADS_ALGO_5	?= $(THREADS)

TITLE_ALGO_0	?= sequential
TITLE_ALGO_1	?= openmp
TITLE_ALGO_2	?= a=2
TITLE_ALGO_3	?= std::thread
TITLE_ALGO_4	?= pthreads
TITLE_ALGO_5	?= boost::thread

INPUT		?= input_qxyz.dat
OUTPUT		?= /dev/null
i		?= 0
comma		?= ,
NUMPROCS	?= 4

run-all: run-all.$(GITREVISION).$(HOSTNAME).$i.log

run-all.$(GITREVISION).$(HOSTNAME).$i.log:
	test ! -f $@.tmp
	@set -x -e; $(foreach r,$(FP_TYPES),$(MAKE) FP_TYPE=$r;)
	@set -x -e; { $(foreach r,$(FP_TYPES),\
		$(foreach a,$(ALGOS),\
		$(foreach t,$(THREADS_ALGO_$a),\
		$(if $(filter-out 0,$t),OMP_NUM_THREADS=$t) ./coulomb.$(r).$(HOSTNAME) $(INPUT) $(OUTPUT) $a 2>&1 || echo FAILED: $$?; \
	))) } | tee $@.tmp
	mv $@.tmp $@

run-all.%.dat: run-all.%.log Makefile
	echo '#COULOMB	GITBRANCH:GITCOMMIT	N	TYPE	ALGO	THREADS	TIME	Ec	rdEc' > $@
	grep '^COULOMB' $< | awk '{ if ($$4 != a) { print "" } }; { a = $$4; print }' >> $@

# only the sequential (threads=1) baseline
run-all.%.dat1: run-all.%.dat
	awk '{ if (/^#|^$$/ || $$6 == 1) { print } }' $< > $@

run-all.%.dat.ps: run-all.%.dat
	a2ps -1 -l110 $< -o $@

run-all.%.dat1.ps: run-all.%.dat1
	a2ps -1 -l110 $< -o $@


run-all.%.gnuplot: run-all.%.dat Makefile
	$(RM) $@
	echo 'set title "$< ($(sort $(shell awk '!/^#/ { print $$2 }' $<))) $(COMMENT)"' >> $@
	echo 'set xlabel "Threads"' >> $@
	echo 'set ylabel "Time [s]"' >> $@
	echo 'plot [] [0:] \\' >> $@
	@set -x -e; \
		$(foreach r,$(FP_TYPES),\
		$(foreach a,$(ALGOS),\
		awk '{ if ($$4 == "$r" && $$5 == $a) { print $$0 } }' $< > ${@:.gnuplot=}.$r.$a.dat ; \
		test ! -s "${@:.gnuplot=}.$r.$a.dat" || echo '	"${@:.gnuplot=}.$r.$a.dat" using 6:7 w $(if $(filter-out 0,$(THREADS_ALGO_$a)),lp,p) title "$(TITLE_ALGO_$a) $r", \\' >> $@ ; \
		))
	echo '	"nothing" notitle' >> $@
	echo 'pause -1' >> $@
	echo 'set term postscript enhanced color solid' >> $@
	echo 'set output "${@:.gnuplot=.time.ps}"' >> $@
	echo 'replot' >> $@
	echo 'set term wxt' >> $@
	echo 'set ylabel "Speedup"' >> $@
	echo 'set key right bottom' >> $@
	echo 'min(a,b) = (a < b) ? a : b' >>$@
	echo 'plot \\' >> $@
	@set -x -e; \
		$(foreach r,$(FP_TYPES),\
		$(foreach a,$(ALGOS),\
		$(if $(filter-out 0,$(THREADS_ALGO_$a)), \
			ref=$$(awk '{ if ($$6 == 1) { print $$7 } }' ${@:.gnuplot=}.$r.$a.dat); \
			test ! -s "${@:.gnuplot=}.$r.$a.dat" || echo '	"${@:.gnuplot=}.$r.$a.dat" using 6:('"$${ref:-0}"'/$$7) w lp title "$(TITLE_ALGO_$a) $r"$(comma) \\' >> $@ ;) \
		))
	echo '	min(x,$(NUMPROCS)) w l ls 0 notitle' >> $@
	echo 'pause -1' >> $@
	echo 'set term postscript enhanced color solid' >> $@
	echo 'set output "${@:.gnuplot=.speedup.ps}"' >> $@
	echo 'replot' >> $@

%.xplot: %.gnuplot
	gnuplot $<

