/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <iostream>
#include "Vec.h"

template <typename T>
void test()
{
    typedef Vec_traits<T> VT;
    typedef typename VT::scalar_type scalar_type;
    VT vt;

    T a = vt.zero();
    T b = vt.same(2);
    T c = vt.same(3);
    scalar_type * pa = reinterpret_cast<scalar_type *>(&a);
    for (int i = 0; i < VT::VecSize; ++i)
        pa[i] = i + 1;

    T d = (a + b) * c;
    T e = c - a;
    T f;
    f = vt.zero();
    e -= d / b;
    f += e;
    f *= a;

    std::cout << __PRETTY_FUNCTION__ << std::endl;
    std::cout << "VecSize                 = " << VT::VecSize << std::endl;
    std::cout << "sizeof(T)               = " << sizeof(T) << std::endl;
    std::cout << "sizeof(VT::scalar_type) = " << sizeof(typename VT::scalar_type) << std::endl;
    std::cout << "sizeof(VT::vec_type)    = " << sizeof(typename VT::vec_type) << std::endl;

    std::cout << VT::VecSize << " * 3 = " << vt.sum(c) << std::endl;
    scalar_type * pd = reinterpret_cast<scalar_type *>(&d);
    for (int i = 0; i < VT::VecSize; ++i)
        std::cout << " " << pd[i];
    std::cout << std::endl;

    std::cout << vt.sum(f) << std::endl;
    std::cout << std::endl;
}

int main()
{
    test<int>();
    typedef int nest0;
    typedef SuperSIMD<nest0, 1> nest1;
    typedef SuperSIMD<nest1, 1> nest2;
    typedef SuperSIMD<nest2, 1> nest3;
    typedef SuperSIMD<nest3, 1> nest4;
    typedef SuperSIMD<nest4, 1> nest5;
    typedef SuperSIMD<nest5, 1> nest6;
    typedef SuperSIMD<nest6, 1> nest7;
    typedef SuperSIMD<nest7, 1> nest8;
    typedef SuperSIMD<nest8, 1> nest9;
    typedef SuperSIMD<nest9, 1> nest10;
    test<nest10>();
#ifdef __AVX__
    test<__m256>();
    test<__m256d>();
    typedef __m256d S;
    typedef SuperSIMD<S, 1> susi1;
    typedef SuperSIMD<S, 2> susi2;
    typedef SuperSIMD<S, 3> susi3;
    typedef SuperSIMD<S, 4> susi4;
    typedef SuperSIMD<susi2, 2> susi22;
    test<susi1>();
    test<susi2>();
    test<susi3>();
    test<susi4>();
    test<susi22>();
#endif
}

// vim: et:ts=4:sw=4
