#ifndef _DaHa_FMM_uninitialized_vector_hpp_
#define _DaHa_FMM_uninitialized_vector_hpp_

#include <cstddef>
#include <memory>

template <typename Tp, typename Allocator = std::allocator<Tp> >
class uninitialized_vector
{
    uninitialized_vector(const uninitialized_vector &) = delete;
    uninitialized_vector &operator=(const uninitialized_vector &) = delete;

public:
    typedef Tp value_type;
    typedef typename Allocator::template rebind<value_type>::other allocator_type;
    typedef size_t size_type;
    typedef Tp *pointer;
    typedef Tp &reference;
    typedef const Tp *const_pointer;
    typedef const Tp &const_reference;
    typedef pointer iterator;
    typedef const_pointer const_iterator;

    uninitialized_vector(size_type n)
            : size_(n)
    {
        array_ = allocator_.allocate(size());
        current_position_ = array_;
    }

    ~uninitialized_vector()
    {
        for (auto it = array_; it != current_position_; ++it)
            allocator_.destroy(it);
        allocator_.deallocate(array_, size());
    }

    template <typename... Args>
    void emplace_back(Args &&... args)
    {
        allocator_.template construct<Tp>(current_position_, std::forward<Args>(args)...);
        ++current_position_;
    }

    size_type size() const { return size_; }

    const_iterator cbegin() const { return array_; }

    const_iterator begin() const { return cbegin(); }

    iterator begin() { return array_; }

    const_iterator cend() const { return array_ + size(); }

    const_iterator end() const { return cend(); }

    iterator end() { return array_ + size(); }

    const_reference operator[](size_type i) const { return array_[i]; }

    reference operator[](size_type i) { return array_[i]; }

    const_pointer data() const { return array_; }

    pointer data() { return array_; }

private:
    pointer array_;
    allocator_type allocator_;
    size_type size_;
    iterator current_position_;
};

#endif
// vim: et:ts=4:sw=4
