/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#include <gtest/gtest.h>

#include "fmsolvr/tree/boxid.hpp"
#include "fmsolvr/tree/boxid_4d.hpp"

namespace  // anonymous namespace
{

template <typename T>
void unused(const T&)
{ }

template <typename BoxIDImpl>
void consume(const fmsolvr::BoxID<BoxIDImpl> &bid)
{
    unused(bid);
}

template <typename Int>
void f()
{
    typedef fmsolvr::BoxID<fmsolvr::BoxID4D<Int> > BoxID;
    BoxID b(1, 1, 1, 1);
    BoxID p = b.parent();
    BoxID c = b.child(0, 1, 0);
    BoxID d = b.descendant(1, 2, 3, 2);
    unused(b.max_depth());
    unused(b.i());
    unused(b.j());
    unused(b.k());
    unused(b.d());
    unused(b.sfcindex());
    ASSERT_TRUE(b != c);
    ASSERT_TRUE(b == b.child(1, 0, 1).parent());
    consume(b);
    consume(c);
    consume(d);
    consume(p);
    consume(b.parent());
    consume(b.child(0, 0, 0));
    consume(b.child(1, 1, 1));
    consume(b.descendant(1, 2, 3, 2));
}

}  // anonymous namespace


TEST(BoxIDTests, InstantiateBoxID) {
    f<uint16_t>();
    f<uint32_t>();
    f<uint64_t>();
    ASSERT_TRUE(1);
}
// vim: et:ts=4:sw=4
