/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <gtest/gtest.h>

#include "fmsolvr/complex_vec.hpp"
#include "fmsolvr/math/Complex.hpp"
#include <complex>
#include <sstream>

namespace  // anonymous namespace
{

template <typename T>
void unused(const T&)
{ }

template <typename Complex>
void f()
{
    typedef typename Complex::value_type Scalar;

    Complex a;
    Scalar re = a.real();
    Scalar im = a.imag();
    Scalar s = re + im;

    a = Complex();
    a = Complex(a);
    a = Complex(re);
    a = Complex(re, im);

    Complex b = a;
    a = b;
    b += a;
    b -= a;
    b *= a;

    -a;
    a + b;
    a - b;
    a * b;
    s * b;
    a * s;

    unused(a == b);
    unused(a != b);

    fmm::abs(a);
    fmm::conj(a);

    std::stringstream os;
    os << a;
}

template <typename Real>
void g()
{
    f<std::complex<Real> >();
    f<fmsolvr::Complex<Real> >();
    f<typename fmsolvr::COMPLEX_GENERATOR<Real>::type>();
}

}  // anonymous namespace


TEST(ComplexTests, InstantiateComplexOperators) {
    g<double>();
    g<float>();
    g<long double>();
    ASSERT_TRUE(1);
}
// vim: et:ts=4:sw=4
