/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <gtest/gtest.h>

#include "fmsolvr/complex_vec.hpp"
#include "fmsolvr/container/triangular_array.hpp"
#include "fmsolvr/fake_stacked_complex_triangle.hpp"
#include "fmsolvr/multipole.hpp"


namespace  // anonymous namespace
{

template <typename TriangularArray>
void access_upper(TriangularArray & t)
{
    typedef typename TriangularArray::value_type value_type;

    const TriangularArray & ct = t;
    value_type v = t.upper(0, 0);
    value_type v2 = ct.upper(t.p(), ct.p());

    t.invalidate_lower();
    t.upper(0, 0) = v;
    t.upper(0, 0) += v2;
}

template <typename ComplexTriangularArray>
void f(size_t p)
{
    typedef fmsolvr::FakeStackedComplexTriangle<1, ComplexTriangularArray> Stacked1;
    typedef fmsolvr::FakeStackedComplexTriangle<2, ComplexTriangularArray> Stacked2;
    typedef fmsolvr::FakeStackedComplexTriangle<1, Stacked1> Stacked1Stacked1;

    ComplexTriangularArray a(p);
    ComplexTriangularArray b(p);

    ComplexTriangularArray * array[2] = {&a, &b};
    Stacked1 s1(array);
    Stacked2 s2(array);

    Stacked1 * sarray[1] = {&s1};
    Stacked1Stacked1 s1s1(sarray);

    access_upper(a);
    access_upper(b);

    access_upper(s1);
    access_upper(s2);

    access_upper(s1s1);
}

template <typename Real>
void g(size_t p)
{
    using namespace fmsolvr;

    f<TriangularArray<typename COMPLEX_GENERATOR<Real>::type, void> >(p);
    f<UpperTriangularArray<typename COMPLEX_GENERATOR<Real>::type, void> >(p);
    f<MultipoleCoefficientsUpper<Real> >(p);
    f<MultipoleCoefficientsUpperLower<Real> >(p);
}

}  // anonymous namespace


TEST(TriangularArrayTests, InstantiateFakeStackedComplexTriangle)
{
    size_t p = 10;
    g<double>(p);
    g<float>(p);
    ASSERT_TRUE(1);
}
// vim: et:ts=4:sw=4
