/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <gtest/gtest.h>

#include "fmsolvr/util/TimeLine.hpp"
#include "fmsolvr/util/wrpapi.hpp"


namespace  // anonymous namespace
{

fmsolvr::papiwrapper<> pw;

enum class Act
{
    FOO,
    BAR,
    _last_enum
};

template <typename TimeLine>
void f(TimeLine &tl)
{
    tl.clear();
    tl.reserve(42);
    auto a = tl.startActivity(Act::FOO);
    tl.stopActivity(a, 42);
    a = tl.startActivity(Act::BAR);
    tl.stopActivity(a, 23);
    a = tl.startActivity(Act::FOO);
    a.changeActivity(Act::BAR);
    tl.stopActivity(a, 13);

    tl.getSize();
}

template <typename TimeLine>
void g()
{
    TimeLine tl;
    f(tl);
    TimeLine tlp(&pw);
    f(tlp);
}

}  // anonymous namespace


TEST(TimeLineTests, InstantiateTimeLine)
{
    g<fmsolvr::timeline::TimeLineNone<Act> >();
    g<fmsolvr::timeline::TimeLineAccumulated<Act> >();
    g<fmsolvr::timeline::TimeLineDetailed<Act> >();
    g<fmsolvr::timeline::TimeLinePapiPrint<Act> >();
    ASSERT_TRUE(1);
}

// vim: et:ts=4:sw=4
