/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <gtest/gtest.h>

#include <immintrin.h>

/*
 * Note that the _mm*_set_p?() intrinsics are confusing, the *last*
 * parameter is element 0, i.e. the %xmm? content in scalar (s?) context.

// Create the vector [A B C D].
//           element  3 2 1 0
__m256d _mm256_set_pd (double A, double B, double C, double D)
{ 
      return (__m256d){ D, C, B, A };
}

 */

#ifdef __AVX__
TEST(SIMDTestCase, AVX256double) {
    int bytesz = 4 * sizeof(double);
    double array[4] = {1, 2, 3, 4};
    __m256d simd = {1, 2, 3, 4};
    __m256d simd_ptr_load = *reinterpret_cast<const __m256d *>(array);
    __m256d simd_load = _mm256_load_pd(array);
    __m256d simd_set_ary = _mm256_set_pd(array[3], array[2], array[1], array[0]);
    __m256d simd_set = _mm256_set_pd(1, 2, 3, 4);
    __m256d simd_set_bw = _mm256_set_pd(4, 3, 2, 1);
    __m256d simd_setr = _mm256_setr_pd(1, 2, 3, 4);

    ASSERT_EQ(0, memcmp(&simd, &simd_ptr_load, bytesz));
    ASSERT_EQ(0, memcmp(&simd, &simd_load, bytesz));
    ASSERT_EQ(0, memcmp(&simd, &simd_set_ary, bytesz));
    ASSERT_NE(0, memcmp(&simd, &simd_set, bytesz));
    ASSERT_EQ(0, memcmp(&simd, &simd_set_bw, bytesz));
    ASSERT_EQ(0, memcmp(&simd, &simd_setr, bytesz));
}
#endif

// vim: et:ts=4:sw=4
