/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <gtest/gtest.h>

#include "operator_testdata_arbitrary.hpp"

#include "fmsolvr/multipole.hpp"
#include "fmsolvr/operator/lattice/latticeoperator.hpp"
#include "fmsolvr/operator/p4/multipole2local.hpp"


namespace  // anonymous namespace
{

template <typename CoefficientMatrix>
void f(size_t p)
{
    using namespace fmsolvr;

    OperatorTestDataArbitrary<CoefficientMatrix> a(p);
    M2L_Operator<CoefficientMatrix> B(a.pos1, a.pos2, p);
    CoefficientMatrix L(2 * p);
    LatticeOperator3D(a.abc, a.ws, L, p, typename CoefficientMatrix::value_type::value_type(1.), 42);
    L.populate_lower();

    M2L_reference_DONOTUSE(a.omega0, a.pos1 + a.pos2, a.mu1, p);
    M2L_reference(a.omega0, a.pos1, a.mu1, a.pos2, p);
    M2L_reference(a.omega0, B, a.mu1, p);
    rawM2L_reference(a.omega0, B, a.mu1, p);
    single_M2L(a.omega0, L, a.mu1, p);
    single_rawM2L(a.omega0, L, a.mu1, p);
}

template <typename Real>
void g(size_t p)
{
    f<fmsolvr::MultipoleCoefficientsUpper<Real> >(p);
    f<fmsolvr::MultipoleCoefficientsUpperLower<Real> >(p);
}

}  // anonymous namespace


TEST(P4Tests, InstantiateM2L)
{
    size_t p = 10;
    g<double>(p);
    g<float>(p);
    g<long double>(p);
    ASSERT_TRUE(1);
}

// vim: et:ts=4:sw=4
