/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <gtest/gtest.h>

#define M2LP3_IMPL_DUMP

#include "rotation_testhelpers.hpp"
#include "rotation_testdata_arbitrary.hpp"

#include "fmsolvr/operator/p3/multipole2local_p3.hpp"
#include "fmsolvr/util/envtools.hpp"


namespace  // anonymous namespace
{

struct test_M2Lp3
{
template <typename RotationDataTypes,
          typename CoefficientMatrix,
          template <typename> class SIMDengine>
void operator()(size_t p)
{
    RotationTestDataArbitrary<RotationDataTypes, CoefficientMatrix> a(p);

    using namespace fmsolvr;

    M2Lp3(a.omega0, a.pos1, a.mu1, a.pos2, p, a.scratch);
    M2Lp3<SIMDengine>(a.omega0, a.RB, a.B, a.mu1, p, a.scratch);

    M2Lp3_impl_default<SIMDengine>(a.omega0, a.RB, a.B, a.mu1, p, a.scratch);
    M2Lp3_impl_reference(a.omega0, a.RB, a.B, a.mu1, p);
    M2Lp3_impl_folded<SIMDengine>(a.omega0, a.RB, a.B, a.mu1, p, a.scratch);
    if (get_env_int("INSTANTIATE_NOISY_CLASSES", 0))
    {
        M2Lp3_impl_folded_dump<SIMDengine>(a.omega0, a.RB, a.B, a.mu1, p, a.scratch);
    }

    M2Lp3_apply_B(a.omega0, a.B, a.mu1, p);
    M2Lp3_apply_B_mlcached<SIMDengine>(a.omega0, a.B, a.mu1, p, a.scratch.array());
}
};

}  // anonymous namespace


TEST(P3Tests, InstantiateM2Lp3)
{
    size_t p = 10;
    test_all_triangle_types(test_M2Lp3(), p);
    ASSERT_TRUE(1);
}

// vim: et:ts=4:sw=4
