/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <gtest/gtest.h>

#include "operator_testdata_arbitrary.hpp"

#include "fmsolvr/multipole.hpp"
#include "fmsolvr/operator/p4/multipole2multipole.hpp"


namespace  // anonymous namespace
{

template <typename CoefficientMatrix>
void f(size_t p)
{
    using namespace fmsolvr;

    OperatorTestDataArbitrary<CoefficientMatrix> a(p);
    M2M_Operator<CoefficientMatrix> A(a.pos1, a.pos2, p);

    M2M(a.omega0, a.pos1, a.omega1, a.pos2, p);
    M2M(a.omega0, A, a.omega1, p);
    M2M_impl_reference(a.omega0, A, a.omega1, p);
}

template <typename Real>
void g(size_t p)
{
    f<fmsolvr::MultipoleCoefficientsUpper<Real> >(p);
    f<fmsolvr::MultipoleCoefficientsUpperLower<Real> >(p);
}

}  // anonymous namespace


TEST(P4Tests, InstantiateM2M)
{
    size_t p = 10;
    g<double>(p);
    g<float>(p);
    g<long double>(p);
    ASSERT_TRUE(1);
}

// vim: et:ts=4:sw=4
