/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <gtest/gtest.h>

#include "rotation_testhelpers.hpp"
#include "rotation_testdata_arbitrary.hpp"

#include "fmsolvr/operator/p3/multipole2multipole_p3.hpp"


namespace  // anonymous namespace
{

struct test_M2Mp3
{
template <typename RotationDataTypes,
          typename CoefficientMatrix,
          template <typename> class SIMDengine>
void operator()(size_t p)
{
    RotationTestDataArbitrary<RotationDataTypes, CoefficientMatrix> a(p);

    using namespace fmsolvr;

    M2Mp3(a.omega0, a.pos1, a.omega1, a.pos2, p, a.scratch);
    M2Mp3<SIMDengine>(a.omega0, a.RAC, a.A, a.omega1, p, a.scratch);

    M2Mp3_impl_default<SIMDengine>(a.omega0, a.RAC, a.A, a.omega1, p, a.scratch);
    M2Mp3_impl_reference(a.omega0, a.RAC, a.A, a.omega1, p);
    M2Mp3_impl_reference_functor(a.omega0, a.RAC, a.A, a.omega1, p);
    M2Mp3_impl_precomp<SIMDengine>(a.omega0, a.RAC, a.A, a.omega1, p, a.scratch);
    M2Mp3_impl_folded<SIMDengine>(a.omega0, a.RAC, a.A, a.omega1, p, a.scratch);

    M2Mp3_apply_A(a.omega0, a.A, a.omega1, p);
    M2Mp3_apply_A_ml<SIMDengine>(a.omega0, a.A, a.omega1, p);
}
};

}  // anonymous namespace


TEST(P3Tests, InstantiateM2Mp3)
{
    size_t p = 10;
    test_all_triangle_types(test_M2Mp3(), p);
    ASSERT_TRUE(1);
}

// vim: et:ts=4:sw=4
