/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#ifndef _AnBe_FMSOLVR_unittests__operator_testdata_arbitrary_hpp_
#define _AnBe_FMSOLVR_unittests__operator_testdata_arbitrary_hpp_

#include "fmsolvr/abc.hpp"
#include "fmsolvr/operator/particle2local.hpp"
#include "fmsolvr/operator/particle2multipole.hpp"
#include "fmsolvr/xyz.hpp"


// The charge and coordiante values used in here are arbitrary values
// without mathematical relevance (or even guaranteeing that convergence
// criteria are satisfied).
template <typename CoefficientMatrix>
struct OperatorTestDataArbitrary {
    typedef typename CoefficientMatrix::value_type coefficient_type;
    typedef typename coefficient_type::value_type Real;
    typedef fmsolvr::XYZ<Real> Real3;
	typedef fmsolvr::ABC<Real3> Real33;

    OperatorTestDataArbitrary(size_t p) :
        omega0(p),
        omega1(p),
        mu0(p),
        mu1(p)
    {
        fmsolvr::P2M_nodivtable(q, pos0, omega0, pos1, p);
        omega0.populate_lower();
        fmsolvr::P2L(q, pos0, mu0, pos1, p);
        mu0.populate_lower();
    }

    const size_t ws = 1;

    const Real3 e1 = Real3(1, 0, 0);
    const Real3 e2 = Real3(0, 1, 0);
    const Real3 e3 = Real3(0, 0, 1);
    const Real33 abc = Real33(e1, e2, e3);

    const Real q = Real(.08/15.);
    const Real3 pos0 = Real3(.2, .3, .5);
    const Real3 pos1 = Real3(.7, .11, .13);
    const Real3 pos2 = Real3(.17, .19, .23);

    CoefficientMatrix omega0;
    CoefficientMatrix omega1;
    CoefficientMatrix mu0;
    CoefficientMatrix mu1;
};

#endif
// vim: et:ts=4:sw=4
