#include <gtest/gtest.h>
#include <iterator>

#include "TreePartition.hpp"
#include "fmsolvr/tree/boxid.hpp"
#include "fmsolvr/tree/boxid_4d.hpp"

template <typename T>
void unused(const T&)
{ }

TEST(TreePartition, basic_construct_and_compare)
{
    typedef fmsolvr::BoxID<fmsolvr::BoxID4D<uint32_t>> BoxID;

    TreePartition<BoxID> tp1(1, 2);
    TreePartition<BoxID> tp2(1, 2, 2);

    TreePartition<BoxID> tp3(1, 1, 2);

    EXPECT_TRUE(tp3.empty());
    EXPECT_FALSE(tp2.empty());

    EXPECT_EQ(tp1, tp2);
    EXPECT_FALSE(tp1 != tp2);

    auto br1 = tp1.box_range();
    auto br2 = tp2.box_range();

    auto begin1 = br1.begin();
    auto begin2 = br2.begin();

    EXPECT_EQ(begin1, begin2);
    ++begin1;
    EXPECT_NE(begin1, begin2);
    EXPECT_EQ(1, br1.size());
    EXPECT_EQ(1, br2.size());
    EXPECT_EQ(0, tp3.box_range().size());

    int i = 0;
    for (auto elem : tp3.box_range())
    {
        unused(elem);
        i++;
    }
    EXPECT_EQ(0, i);
    for (auto elem : tp2.box_range())
    {
        unused(elem);
        i++;
    }
    EXPECT_EQ(1, i);

    BoxID bid1(1, 2);
    EXPECT_EQ(*(tp2.box_range().begin()), bid1);
}
