/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#include <gtest/gtest.h>

#include "rotation_testhelpers.hpp"
#include "rotation_testdata_arbitrary.hpp"

#include "fmsolvr/operator/rotation/rotation.hpp"


namespace  // anonymous namespace
{

struct test_rotation
{
template <typename RotationDataTypes,
          typename CoefficientMatrix,
          template <typename> class SIMDengine>
void operator()(size_t p)
{
    RotationTestDataArbitrary<RotationDataTypes, CoefficientMatrix> a(p);

    using namespace fmsolvr;

    // _reference
    ApplyRotationMultipoleForward_reference(a.omega0, a.RAC, a.omega1, p);
    ApplyRotationMultipoleBackward_reference(a.omega0, a.RAC, a.omega1, p);
    ApplyRotationLocalForward_reference(a.mu0, a.RAC, a.mu1, p);
    ApplyRotationLocalBackward_reference(a.mu0, a.RAC, a.mu1, p);

    // _reference + scratch
    ApplyRotationMultipoleForward_reference(a.omega0, a.RAC, a.omega1, p, a.scratch.array());
    ApplyRotationMultipoleBackward_reference(a.omega0, a.RAC, a.omega1, p, a.scratch.array());
    ApplyRotationLocalForward_reference(a.mu0, a.RAC, a.mu1, p, a.scratch.array());
    ApplyRotationLocalBackward_reference(a.mu0, a.RAC, a.mu1, p, a.scratch.array());

    // (empty)
    ApplyRotationMultipoleForward(a.omega0, a.RAC, a.omega1, p);
    ApplyRotationMultipoleBackward(a.omega0, a.RAC, a.omega1, p);
    ApplyRotationLocalForward(a.mu0, a.RAC, a.mu1, p);
    ApplyRotationLocalBackward(a.mu0, a.RAC, a.mu1, p);

    // (empty) + scratch
    ApplyRotationMultipoleForward(a.omega0, a.RAC, a.omega1, p, a.scratch.array());
    ApplyRotationMultipoleBackward(a.omega0, a.RAC, a.omega1, p, a.scratch.array());
    ApplyRotationLocalForward(a.mu0, a.RAC, a.mu1, p, a.scratch.array());
    ApplyRotationLocalBackward(a.mu0, a.RAC, a.mu1, p, a.scratch.array());

    // _phi
    ApplyRotationMultipoleForward_phi(a.omega0, a.RAC, a.omega1, p);
    ApplyRotationMultipoleBackward_phi(a.omega0, a.RAC, a.omega1, p);
    ApplyRotationLocalForward_phi(a.mu0, a.RAC, a.mu1, p);
    ApplyRotationLocalBackward_phi(a.mu0, a.RAC, a.mu1, p);

    // _phi + scratch
    ApplyRotationMultipoleForward_phi(a.omega0, a.RAC, a.omega1, p, a.scratch.array());
    ApplyRotationMultipoleBackward_phi(a.omega0, a.RAC, a.omega1, p, a.scratch.array());
    ApplyRotationLocalForward_phi(a.mu0, a.RAC, a.mu1, p, a.scratch.array());
    ApplyRotationLocalBackward_phi(a.mu0, a.RAC, a.mu1, p, a.scratch.array());

    // _phi_f/g
    ApplyRotationMultipoleForward_phi_f(a.omega0, a.RAC, a.omega1, p);
    ApplyRotationMultipoleBackward_phi_f(a.omega0, a.RAC, a.omega1, p);
    ApplyRotationLocalForward_phi_g(a.mu0, a.RAC, a.mu1, p);
    ApplyRotationLocalBackward_phi_g(a.mu0, a.RAC, a.mu1, p);

    // _phi_f/g + scratch
    ApplyRotationMultipoleForward_phi_f(a.omega0, a.RAC, a.omega1, p, a.scratch.array());
    ApplyRotationMultipoleBackward_phi_f(a.omega0, a.RAC, a.omega1, p, a.scratch.array());
    ApplyRotationLocalForward_phi_g(a.mu0, a.RAC, a.mu1, p, a.scratch.array());
    ApplyRotationLocalBackward_phi_g(a.mu0, a.RAC, a.mu1, p, a.scratch.array());

    // _folded
    //ApplyRotationMultipoleForward_folded<SIMDengine>(a.omega0, a.RAC, a.omega1, p);
    ApplyRotationMultipoleBackward_folded<SIMDengine>(a.omega0, a.RAC, a.omega1, p);
    //ApplyRotationLocalForward_folded<SIMDengine>(a.mu0, a.RAC, a.mu1, p);
    ApplyRotationLocalBackward_folded<SIMDengine>(a.mu0, a.RAC, a.mu1, p);

    // _folded + scratch
    ApplyRotationMultipoleForward_folded<SIMDengine>(a.omega0, a.RAC, a.omega1, p, a.scratch.array());
    ApplyRotationMultipoleBackward_folded<SIMDengine>(a.omega0, a.RAC, a.omega1, p, a.scratch.array());
    ApplyRotationLocalForward_folded<SIMDengine>(a.mu0, a.RAC, a.mu1, p, a.scratch.array());
    ApplyRotationLocalBackward_folded<SIMDengine>(a.mu0, a.RAC, a.mu1, p, a.scratch.array());
}
};

}  // anonymous namespace


TEST(P3Tests, InstantiateRotate)
{
    size_t p = 10;
    test_all_triangle_types(test_rotation(), p);
    ASSERT_TRUE(1);
}

// vim: et:ts=4:sw=4
