/*
 * This file is part of FMSOLVR, a Fast Multipole Solver for MD.
 *
 * Copyright © 2019 Andreas Beckmann
 *
 * FMSOLVR is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation, either version 2.1
 * of the License, or (at your option) any later version.
 *
 * FMSOLVR is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with FMSOLVR.  If not, see
 * <https://www.gnu.org/licenses/>.
 */
#ifndef _AnBe_FMM_unittests__simd_real_helpers_hpp_
#define _AnBe_FMM_unittests__simd_real_helpers_hpp_

#include "fmsolvr/simd/simd_engine.hpp"
#include "fmsolvr/util/float128.hpp"


namespace  // anonymous namespace
{

template <typename Real, template <typename> class SIMDengine, typename Functor, typename... Args>
void run_functor(Functor f, Args... args)
{
    // two levels of SIMDengine rebinding indirection to solve default => impl => none
    f.template operator()<Real, SIMDengine<Real>::template engine<Real>::template engine>(args...);
}

template <typename Real, typename Functor, typename... Args>
void test_all_simd_engines(Functor f, Args... args)
{
    using namespace fmsolvr;

    run_functor<Real, simd_engine_default>(f, args...);
    run_functor<Real, simd_engine_none>(f, args...);
    run_functor<Real, simd_engine_array1>(f, args...);
    run_functor<Real, simd_engine_array2>(f, args...);
    run_functor<Real, simd_engine_array4>(f, args...);
    run_functor<Real, simd_engine_array8>(f, args...);
#ifdef FMSOLVR_HAVE_SSE
    run_functor<Real, simd_engine_sse>(f, args...);
#endif
#ifdef FMSOLVR_HAVE_AVX
    run_functor<Real, simd_engine_avx>(f, args...);
#endif
#ifdef FMSOLVR_HAVE_AVX512F
    run_functor<Real, simd_engine_avx512>(f, args...);
#endif
}

template <typename Functor, typename... Args>
void test_all_real_types(Functor f, Args... args)
{
    test_all_simd_engines<double>(f, args...);
    test_all_simd_engines<float>(f, args...);
    test_all_simd_engines<long double>(f, args...);
#if defined(HAVE_FLOAT128)
    test_all_simd_engines<__float128>(f, args...);
#endif
}

}  // anonymous namespace

#endif
// vim: et:ts=4:sw=4
