#ifndef _AnBe_FMM_pass3_hpp_
#define _AnBe_FMM_pass3_hpp_

#include "fmsolvr/ijk.hpp"
#include "fmsolvr/operator/p3/local2local_p3.hpp"
#include "fmsolvr/operator/p4/local2local.hpp"
#include "fmsolvr/util/envtools.hpp"

namespace fmsolvr {

template <
        typename FMMHandle,
        typename TaskFactory,
        typename MutexContainer,
        typename DependencyManager>
void pass3_L2L(
        FMMHandle &fmm_handle,
        size_t id,
        size_t d,
        TaskFactory &task_factory,
        MutexContainer &mutex_container,
        DependencyManager &dm)
{
    const bool p4 = get_env_int("P4", 0);
    typedef typename FMMHandle::real3_type Real3;
    typedef typename FMMHandle::size_type size_type;
    typedef typename FMMHandle::boxid_type BoxID;
    typedef typename FMMHandle::coefficient_type coefficient_type;
    typedef typename FMMHandle::RotationDataTypes::Rot_Operator RotationOperator;
    typedef typename FMMHandle::RotationDataTypes::L2L_Operator L2Lp3_Operator;
    typedef typename FMMHandle::RotationDataTypes::template ScratchTypeGenerator<coefficient_type>::Scratch scratch_type;

    const auto &simulationbox = fmm_handle.simulationbox();
    const auto &tree_omega = fmm_handle.tree_omega();
    auto & tree_mu = fmm_handle.tree_mu();
    auto &G = fmm_handle.RotationGeneratorL2L();
    const size_type p = fmm_handle.p();
    scratch_type scratch(p);

    // parent = source
    // child = target
    {
        size_t d_p = d;
        size_t d_c = d_p + 1;

        size_t d_delta = d_c - d_p;
        size_t dim_op = size_t(1) << d_delta;
        const auto & l2l_operator = fmm_handle.l2l_operator_cube(d_c, d_delta);
        {
            {
                {

                    BoxID bid_p(id, d_p);
                    Real3 expn_point_p = simulationbox.expansion_point(bid_p);
                    const auto & mu_p = tree_mu[bid_p];
                    for (size_t ii = 0; ii < dim_op; ++ii) {
                        for (size_t jj = 0; jj < dim_op; ++jj) {
                            for (size_t kk = 0; kk < dim_op; ++kk) {
                                BoxID bid_c = bid_p.descendant(ii, jj, kk, d_delta);
                                // no particles in the target box?
                                // (this HACK only works if source tree and target tree are
                                // identical)
                                if (tree_omega[bid_c].empty())
                                    continue;
                                IJK<size_t> ijk_c(ii, jj, kk);
                                Real3 expn_point_c = simulationbox.expansion_point(bid_c);
                                //L2L operator similar to M2M operator -> exploit parallelism to achieve good memory access pattern (see above)
                                auto & tree_ow_mu = mutex_container.tree_ow_;
                                auto & ow = tree_ow_mu[bid_c];

                                {
                                auto reservation = ow.AcquireAccess();
                                auto & tmp_mu_c = reservation.Obj();
                                if (p4) {
                                    L2L(mu_p, l2l_operator(ii, jj, kk), tmp_mu_c, p);
                                } else {
                                    RotationOperator R =
                                            G.generate_rotation_operator(ijk_c, d_delta);
                                    L2Lp3_Operator C_p3(expn_point_p, expn_point_c, p);
                                    L2Lp3<FMMHandle::template SIMDengine>(mu_p, R, C_p3, tmp_mu_c, p, scratch);
                                }
                                tmp_mu_c.addSourceCounters(mu_p.sourceCounters());
                                tmp_mu_c.populate_lower();
                                } // ow.Oap().WithdrawObjectAccess(reservation);
                                dm.ResolvePartialMu(bid_c, task_factory, tree_ow_mu);
                            }
                        }
                    }
                }
            }
        }
    }
}

}  // namespace fmsolvr

#endif
// vim: et:ts=4:sw=4
